// RubiksCube.cpp: implementation of the CRubiksCube class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RubiksCube.h"
#include "Defines.h"
#include <stdlib.h>
#include <time.h>
#ifdef _DEBUG
#include <stdio.h>
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//{FRONT, RIGHT, LEFT, BACK,	TOP,	BOTTOM};
//{WHITE, GREEN, BLUE, YELLOW,	RED,	ORANGE, BLACK};
const int CRubiksCube::m_nSolvedCube[6][DIMENSIONS][DIMENSIONS] = 
{
	{
		{WHITE,WHITE,WHITE},
		{WHITE,WHITE,WHITE},
		{WHITE,WHITE,WHITE}
	},
	{
		{GREEN, GREEN, GREEN},
		{GREEN, GREEN, GREEN},
		{GREEN, GREEN, GREEN}
	},
	{
		{BLUE, BLUE, BLUE},
		{BLUE, BLUE, BLUE},
		{BLUE, BLUE, BLUE}
	},
	{
		{YELLOW, YELLOW, YELLOW},
		{YELLOW, YELLOW, YELLOW},
		{YELLOW, YELLOW, YELLOW}
	},
	{
		{RED, RED, RED},
		{RED, RED, RED},
		{RED, RED, RED}
	},
	{
		{ORANGE, ORANGE, ORANGE},
		{ORANGE, ORANGE, ORANGE},
		{ORANGE, ORANGE, ORANGE}
	},
};

const	int	CRubiksCube::WHITEFACE	=	0x01;
const	int	CRubiksCube::BLUEFACE	=	0x02;
const	int	CRubiksCube::REDFACE	=	0x04;
const	int	CRubiksCube::ORANGEFACE	=	0x08;
const	int	CRubiksCube::YELLOWFACE	=	0x10;
const	int	CRubiksCube::GREENFACE	=	0x20;

const float CRubiksCube::m_fCubieOffsetFromCenter = (float) ((LENGTHOFCUBESIDE / DIMENSIONS) - HALFLENGTHOFCUBIESIDE);

CRubiksCube::CRubiksCube()
{
	srand((unsigned)time(NULL));
	ResetCube();
	m_fRotationSpeed	=	ROTATIONSPEED;
}

CRubiksCube::~CRubiksCube()
{
	m_ListOfMoves.clear();
}

void CRubiksCube::ScrambleCube()
{
	//100 random twists
	int nNumTwists;
#ifdef _DEBUG
	nNumTwists = 30;
#else
	nNumTwists = 100;
#endif
	int nLastFaceChosen = -1;
	int nFace = -1;
	int nTruth= -1;
	bool bLastClockwise = false;
	bool bClockWise = false;
	for (int x = 0; x < nNumTwists; x++)
	{
		CMove aMove;
		do
		{
			nFace = (rand() % (BOTTOM + 1));
			nTruth= (rand() % 100) + 1;
			bClockWise = ((nTruth % 2) == 0) ? true : false;
		}
		while (nFace == nLastFaceChosen && bLastClockwise == !bClockWise);
		
		bLastClockwise = bClockWise;
		nLastFaceChosen = nFace;
		RotateFace(nFace, bClockWise);
	}
}

void CRubiksCube::Draw()
{
	float fXAxis = 0.0f;
	float fYAxis = 0.0f;
	float fZAxis = 0.0f;

	glRotatef(45, 1.0f, 1.0f, 0.0f);
	glRotatef(m_fXRotation, 1.0f, 0.0f, 0.0f);
	glRotatef(m_fYRotation, 0.0f, 1.0f, 0.0f);
	glRotatef(m_fZRotation, 0.0f, 0.0f, 1.0f);
	//reset all squares to NOT YET DRAWN
	ResetDrawnStatus();

	//set axis of rotation
	if (FRONT == m_nRotatingFace || BACK == m_nRotatingFace)
	{
		//z axis
		fZAxis = 1.0f;
	}
	else if(RIGHT == m_nRotatingFace || LEFT == m_nRotatingFace)
	{
		//x axis
		fXAxis = 1.0f;
	}
	else if(TOP == m_nRotatingFace || BOTTOM == m_nRotatingFace)
	{
		//y axis
		fYAxis = 1.0f;
	}

	glPushMatrix();
	switch(m_nRotatingFace)
	{
		case FRONT:
			{
				if(!m_bClockwise)
					m_fZFaceRotation += m_fRotationSpeed;
				else
					m_fZFaceRotation -= m_fRotationSpeed;

				if(m_fZFaceRotation > 90 || m_fZFaceRotation < -90)
					m_fZFaceRotation = 0.0f;

				glRotatef(m_fZFaceRotation, fXAxis, fYAxis, fZAxis);

				if(0.0f == m_fZFaceRotation)
				{
					RotateFront(FRONT, m_bClockwise);
#ifdef _DEBUG
					OutputCube();
#endif
					m_nRotatingFace =	-1;//set an invalid rotating face
				}

				//draw all the front faces that are rotating
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawFront(FRONT, nRow, nCol);
					}
				}
				//now draw the attached squares
				DrawTop(TOP, BOTTOMROW, LEFTCOL);
				DrawTop(TOP, BOTTOMROW, MIDDLECOL);
				DrawTop(TOP, BOTTOMROW, RIGHTCOL);

				DrawRight(RIGHT, TOPROW, LEFTCOL);
				DrawRight(RIGHT, MIDDLEROW, LEFTCOL);
				DrawRight(RIGHT, BOTTOMROW, LEFTCOL);

				DrawLeft(LEFT, TOPROW, RIGHTCOL);
				DrawLeft(LEFT, MIDDLEROW, RIGHTCOL);
				DrawLeft(LEFT, BOTTOMROW, RIGHTCOL);

				DrawBottom(BOTTOM, TOPROW, LEFTCOL);
				DrawBottom(BOTTOM, TOPROW, MIDDLECOL);
				DrawBottom(BOTTOM, TOPROW, RIGHTCOL);
			}
			break;
		case LEFT:
			{
				if(!m_bClockwise)
					m_fXFaceRotation -= m_fRotationSpeed;
				else
					m_fXFaceRotation += m_fRotationSpeed;

				if(m_fXFaceRotation > 90 || m_fXFaceRotation < -90)
					m_fXFaceRotation = 0.0f;

				glRotatef(m_fXFaceRotation, fXAxis, fYAxis, fZAxis);
				if(0.0f == m_fXFaceRotation)
				{
					RotateLeft(LEFT, m_bClockwise);
#ifdef _DEBUG
					OutputCube();
#endif
					m_nRotatingFace =	-1;//set an invalid rotating face
				}

				//draw all the left faces that are rotating
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawLeft(LEFT, nRow, nCol);
					}
				}
				//now draw the attached squares
				DrawTop(TOP, TOPROW, LEFTCOL);
				DrawTop(TOP, MIDDLEROW, LEFTCOL);
				DrawTop(TOP, BOTTOMROW, LEFTCOL);

				DrawFront(FRONT, TOPROW, LEFTCOL);
				DrawFront(FRONT, MIDDLEROW, LEFTCOL);
				DrawFront(FRONT, BOTTOMROW, LEFTCOL);

				DrawBottom(BOTTOM, TOPROW, LEFTCOL);
				DrawBottom(BOTTOM, MIDDLEROW, LEFTCOL);
				DrawBottom(BOTTOM, BOTTOMROW, LEFTCOL);

				DrawBack(BACK, TOPROW, RIGHTCOL);
				DrawBack(BACK, MIDDLEROW, RIGHTCOL);
				DrawBack(BACK, BOTTOMROW, RIGHTCOL);
			}
			break;
		case BACK:
			{
				//Drawn from the perspective of FRONT face
				//so the clockwise values must be reversed
				if(!m_bClockwise)
					m_fZFaceRotation -= m_fRotationSpeed;
				else
					m_fZFaceRotation += m_fRotationSpeed;

				if(m_fZFaceRotation > 90 || m_fZFaceRotation < -90)
					m_fZFaceRotation = 0.0f;

				glRotatef(m_fZFaceRotation, fXAxis, fYAxis, fZAxis);
				if(0.0f == m_fZFaceRotation)
				{
					RotateBack(BACK, m_bClockwise);
#ifdef _DEBUG
					OutputCube();
#endif
					m_nRotatingFace =	-1;//set an invalid rotating face
				}

				//draw all the right faces that are rotating
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawBack(BACK, nRow, nCol);
					}
				}
				//now draw the attached squares
				DrawTop(TOP, TOPROW, LEFTCOL);
				DrawTop(TOP, TOPROW, MIDDLECOL);
				DrawTop(TOP, TOPROW, RIGHTCOL);

				DrawLeft(LEFT, TOPROW, LEFTCOL);
				DrawLeft(LEFT, MIDDLEROW, LEFTCOL);
				DrawLeft(LEFT, BOTTOMROW, LEFTCOL);

				DrawBottom(BOTTOM, BOTTOMROW, LEFTCOL);
				DrawBottom(BOTTOM, BOTTOMROW, MIDDLECOL);
				DrawBottom(BOTTOM, BOTTOMROW, RIGHTCOL);

				DrawRight(RIGHT, TOPROW, RIGHTCOL);
				DrawRight(RIGHT, MIDDLEROW, RIGHTCOL);
				DrawRight(RIGHT, BOTTOMROW, RIGHTCOL);
			}
			break;
		case RIGHT:
			{
				if(!m_bClockwise)
					m_fXFaceRotation += m_fRotationSpeed;
				else
					m_fXFaceRotation -= m_fRotationSpeed;

				if(m_fXFaceRotation > 90 || m_fXFaceRotation < -90)
					m_fXFaceRotation = 0.0f;
				glRotatef(m_fXFaceRotation, fXAxis, fYAxis, fZAxis);
				if(0.0f == m_fXFaceRotation)
				{
					RotateRight(RIGHT, m_bClockwise);
#ifdef _DEBUG
					OutputCube();
#endif
					m_nRotatingFace =	-1;//set an invalid rotating face
				}

				//draw all the right faces that are rotating
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawRight(RIGHT, nRow, nCol);
					}
				}
				//now draw the attached squares
				DrawTop(TOP, TOPROW, RIGHTCOL);
				DrawTop(TOP, MIDDLEROW, RIGHTCOL);
				DrawTop(TOP, BOTTOMROW, RIGHTCOL);

				DrawFront(FRONT, TOPROW, RIGHTCOL);
				DrawFront(FRONT, MIDDLEROW, RIGHTCOL);
				DrawFront(FRONT, BOTTOMROW, RIGHTCOL);

				DrawBottom(BOTTOM, TOPROW, RIGHTCOL);
				DrawBottom(BOTTOM, MIDDLEROW, RIGHTCOL);
				DrawBottom(BOTTOM, BOTTOMROW, RIGHTCOL);

				DrawBack(BACK, TOPROW, LEFTCOL);
				DrawBack(BACK, MIDDLEROW, LEFTCOL);
				DrawBack(BACK, BOTTOMROW, LEFTCOL);
			}
			break;
		case BOTTOM:
			{
				if(!m_bClockwise)
					m_fYFaceRotation -= m_fRotationSpeed;
				else
					m_fYFaceRotation += m_fRotationSpeed;

				if(m_fYFaceRotation > 90 || m_fYFaceRotation < -90)
					m_fYFaceRotation = 0.0f;
				glRotatef(m_fYFaceRotation, fXAxis, fYAxis, fZAxis);
				if (0.0f == m_fYFaceRotation)
				{
					RotateBottom(BOTTOM, m_bClockwise);
#ifdef _DEBUG
					OutputCube();
#endif
					m_nRotatingFace =	-1;//set an invalid rotating face
				}
				//draw all the right faces that are rotating
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawBottom(BOTTOM, nRow, nCol);
					}
				}
				//now draw the attached squares
				DrawFront(FRONT, BOTTOMROW, LEFTCOL);
				DrawFront(FRONT, BOTTOMROW, MIDDLECOL);
				DrawFront(FRONT, BOTTOMROW, RIGHTCOL);

				DrawLeft(LEFT, BOTTOMROW, LEFTCOL);
				DrawLeft(LEFT, BOTTOMROW, MIDDLECOL);
				DrawLeft(LEFT, BOTTOMROW, RIGHTCOL);

				DrawBack(BACK, BOTTOMROW, LEFTCOL);
				DrawBack(BACK, BOTTOMROW, MIDDLECOL);
				DrawBack(BACK, BOTTOMROW, RIGHTCOL);

				DrawRight(RIGHT, BOTTOMROW, LEFTCOL);
				DrawRight(RIGHT, BOTTOMROW, MIDDLECOL);
				DrawRight(RIGHT, BOTTOMROW, RIGHTCOL);
			}
			break;
		case TOP:
			{
				if(!m_bClockwise)
					m_fYFaceRotation += m_fRotationSpeed;
				else
					m_fYFaceRotation -= m_fRotationSpeed;

				if(m_fYFaceRotation > 90 || m_fYFaceRotation < -90)
					m_fYFaceRotation = 0.0f;
				glRotatef(m_fYFaceRotation, fXAxis, fYAxis, fZAxis);
				if (0.0f == m_fYFaceRotation)
				{
					RotateTop(TOP, m_bClockwise);
#ifdef _DEBUG
					OutputCube();
#endif
					m_nRotatingFace =	-1;//set an invalid rotating face
				}
				//draw all the right faces that are rotating
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawTop(TOP, nRow, nCol);
					}
				}
				//now draw the attached squares
				DrawFront(FRONT, TOPROW, LEFTCOL);
				DrawFront(FRONT, TOPROW, MIDDLECOL);
				DrawFront(FRONT, TOPROW, RIGHTCOL);

				DrawLeft(LEFT, TOPROW, LEFTCOL);
				DrawLeft(LEFT, TOPROW, MIDDLECOL);
				DrawLeft(LEFT, TOPROW, RIGHTCOL);

				DrawBack(BACK, TOPROW, LEFTCOL);
				DrawBack(BACK, TOPROW, MIDDLECOL);
				DrawBack(BACK, TOPROW, RIGHTCOL);

				DrawRight(RIGHT, TOPROW, LEFTCOL);
				DrawRight(RIGHT, TOPROW, MIDDLECOL);
				DrawRight(RIGHT, TOPROW, RIGHTCOL);
			}
			break;
	}//switch(m_nRotatingFace)

	glPopMatrix();
	glPushMatrix();
	for (int currFace = FRONT; currFace <= BOTTOM; currFace++)	//loop through all the faces
	{
		//since we just drew the rotating face first,
		//we want to skip the face we already drew
		if (currFace == m_nRotatingFace)
			continue;
		switch(currFace)
		{
		case FRONT:
			{
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawFront(currFace, nRow, nCol);
					}
				}
			}
			break;
		case LEFT:
			{
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawLeft(currFace, nRow, nCol);
					}
				}
			}
			break;
		case BACK:
			{
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawBack(currFace, nRow, nCol);
					}
				}
			}
			break;
		case RIGHT:
			{
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawRight(currFace, nRow, nCol);
					}
				}
			}
			break;
		case BOTTOM:
			{
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawBottom(currFace, nRow, nCol);
					}
				}
			}
			break;
		case TOP:
			{
				for (int nRow = 0; nRow < DIMENSIONS; nRow++)
				{
					for (int nCol = 0; nCol < DIMENSIONS; nCol++)
					{
						DrawTop(currFace, nRow, nCol);
					}
				}
			}
			break;
		}
	}
	glPopMatrix();	
}

void CRubiksCube::ResetCube()
{
	//Set each squares color, value showing on this face, and all the colors shared by this cubie
	m_Squares[WHITE][0][0].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | REDFACE | BLUEFACE);
	m_Squares[WHITE][0][1].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | REDFACE);
	m_Squares[WHITE][0][2].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | REDFACE | GREENFACE);

	m_Squares[WHITE][1][0].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | BLUEFACE);
	m_Squares[WHITE][1][1].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE);
	m_Squares[WHITE][1][2].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | GREENFACE);

	m_Squares[WHITE][2][0].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | ORANGEFACE | BLUEFACE);
	m_Squares[WHITE][2][1].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | ORANGEFACE);
	m_Squares[WHITE][2][2].SetColor(CColors::GetColor(WHITE), WHITE, WHITEFACE | ORANGEFACE | GREENFACE);

	m_Squares[BLUE][0][0].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | REDFACE | YELLOWFACE);
	m_Squares[BLUE][0][1].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | REDFACE);
	m_Squares[BLUE][0][2].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | REDFACE | WHITEFACE);

	m_Squares[BLUE][1][0].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | YELLOWFACE);
	m_Squares[BLUE][1][1].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE);
	m_Squares[BLUE][1][2].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | WHITEFACE);

	m_Squares[BLUE][2][0].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | ORANGEFACE | YELLOWFACE);
	m_Squares[BLUE][2][1].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | ORANGEFACE);
	m_Squares[BLUE][2][2].SetColor(CColors::GetColor(BLUE), BLUE, BLUEFACE | ORANGEFACE | WHITEFACE);

	m_Squares[YELLOW][0][0].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | REDFACE | GREENFACE);
	m_Squares[YELLOW][0][1].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | REDFACE);
	m_Squares[YELLOW][0][2].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | REDFACE | BLUEFACE);

	m_Squares[YELLOW][1][0].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | GREENFACE);
	m_Squares[YELLOW][1][1].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE);
	m_Squares[YELLOW][1][2].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | BLUEFACE);

	m_Squares[YELLOW][2][0].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | ORANGEFACE | GREENFACE);
	m_Squares[YELLOW][2][1].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | ORANGEFACE);
	m_Squares[YELLOW][2][2].SetColor(CColors::GetColor(YELLOW), YELLOW, YELLOWFACE | ORANGEFACE | BLUEFACE);

	m_Squares[GREEN][0][0].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | REDFACE | WHITEFACE);
	m_Squares[GREEN][0][1].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | REDFACE);
	m_Squares[GREEN][0][2].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | REDFACE | YELLOWFACE);

	m_Squares[GREEN][1][0].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | WHITEFACE);
	m_Squares[GREEN][1][1].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE);
	m_Squares[GREEN][1][2].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | YELLOWFACE);

	m_Squares[GREEN][2][0].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | ORANGEFACE | WHITEFACE);
	m_Squares[GREEN][2][1].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | ORANGEFACE);
	m_Squares[GREEN][2][2].SetColor(CColors::GetColor(GREEN), GREEN, GREENFACE | ORANGEFACE | YELLOWFACE);

	m_Squares[RED][0][0].SetColor(CColors::GetColor(RED), RED, REDFACE | YELLOWFACE | BLUEFACE);
	m_Squares[RED][0][1].SetColor(CColors::GetColor(RED), RED, REDFACE | YELLOWFACE);
	m_Squares[RED][0][2].SetColor(CColors::GetColor(RED), RED, REDFACE | YELLOWFACE | GREENFACE);

	m_Squares[RED][1][0].SetColor(CColors::GetColor(RED), RED, REDFACE | BLUEFACE);
	m_Squares[RED][1][1].SetColor(CColors::GetColor(RED), RED, REDFACE);
	m_Squares[RED][1][2].SetColor(CColors::GetColor(RED), RED, REDFACE | GREENFACE);

	m_Squares[RED][2][0].SetColor(CColors::GetColor(RED), RED, REDFACE | WHITEFACE | BLUEFACE);
	m_Squares[RED][2][1].SetColor(CColors::GetColor(RED), RED, REDFACE | WHITEFACE);
	m_Squares[RED][2][2].SetColor(CColors::GetColor(RED), RED, REDFACE | WHITEFACE | GREENFACE);

	m_Squares[ORANGE][0][0].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | WHITEFACE | BLUEFACE);
	m_Squares[ORANGE][0][1].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | WHITEFACE);
	m_Squares[ORANGE][0][2].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | WHITEFACE | GREENFACE);

	m_Squares[ORANGE][1][0].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | BLUEFACE);
	m_Squares[ORANGE][1][1].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE);
	m_Squares[ORANGE][1][2].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | GREENFACE);

	m_Squares[ORANGE][2][0].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | YELLOWFACE | BLUEFACE);
	m_Squares[ORANGE][2][1].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | YELLOWFACE);
	m_Squares[ORANGE][2][2].SetColor(CColors::GetColor(ORANGE), ORANGE, ORANGEFACE | YELLOWFACE | GREENFACE);

	m_nRotatingFace = -1;
	m_bClockwise = false;
	m_fXFaceRotation = 0.0f;
	m_fYFaceRotation = 0.0f;
	m_fZFaceRotation = 0.0f;

	m_fvPosition[0] =	0.0f;
	m_fvPosition[0] =	0.0f;
	m_fvPosition[0] =	0.0f;
	m_fXRotation	=	0.0f;
	m_fYRotation	=	0.0f;
	m_fZRotation	=	0.0f;
	m_fAlpha		=	1.0f;

	ResetDrawnStatus();
	m_ListOfMoves.clear();
	m_Solution.clear();
	m_ListIterator		=	NULL;
	//m_fRotationSpeed	=	ROTATIONSPEED;
	m_nCubeState		=	ROTATINGFACE;
	m_nFaceToSolve		=	FRONT;
	m_bAutoSolve		=	false;//true;
	m_nPrevCubeState	=	-1;
	ScrambleCube();
	//srand((unsigned)time(NULL));
}

void CRubiksCube::DrawFront(int currFace, int row, int col)
{
	if(m_Squares[currFace][row][col].m_bDrawn)
		return;

	float fXOffsetCenter, fYOffsetCenter, fZOffsetCenter;
	//FOR THE FRONT FACE, WE WANT TO DRAW FROM THE
	//CENTER...SQUARE VALUES
	//	[0][0]	[0][1]	[0][2]	
	//	[1][0]	[1][1]	[1][2]
	//	[2][0]	[2][1]	[2][2]
	//OFFSETS
	//	(-1,1,Z)	(0,1,Z)		(1,1,Z)
	//	(-1,0,Z)	(0,0,Z)		(1,0,Z)
	//	(-1,-1,Z)	(0,-1,Z)	(1,0,Z)

	fXOffsetCenter = LENGTHOFCUBIESIDE * (col - 1);
	fYOffsetCenter = LENGTHOFCUBIESIDE * (-(row - 1));
	fZOffsetCenter = LENGTHOFCUBIESIDE;

	float fvCol[4];
	float* fvPointer;
	//memcpy(fvCol, CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal), sizeof(CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal)));
	fvPointer = CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal);
	for (int a = 0; a < 3; a++)
	{
		fvCol[a] = fvPointer[a];
	}
	fvCol[3] = m_fAlpha;

	glPushMatrix();
	glTranslatef(fXOffsetCenter, fYOffsetCenter, fZOffsetCenter);
	glNormal3f(0.0f, 0.0f, 1.0f);
	glColor4fv(fvCol);
	//DRAW THIS FACE
	glBegin(GL_TRIANGLES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top left
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom left

		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom right
	glEnd();
	//this square was just drawn, so set its value to true
	//to prevent it from tyring to be drawwn again
	m_Squares[currFace][row][col].m_bDrawn = true;
	glColor3fv(CColors::GetColor(BLACK));
	//TRACE EACH EDGE OF EACH CUBIE TO
	//MAKE A DISTINCTION BETWEEN EACH PIECE
	glBegin(GL_LINES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
	glEnd();
	
	glPopMatrix();
}

void CRubiksCube::DrawLeft(int currFace, int row, int col)
{
	//if we have drawn this square already, move to
	//the next square
	if(m_Squares[currFace][row][col].m_bDrawn)
		return;

	float fXOffsetCenter, fYOffsetCenter, fZOffsetCenter;
	//FOR THE LEFT FACE, WE WANT TO DRAW FROM THE
	//CENTER...SQUARE VALUES
	//	[0][0]	[0][1]	[0][2]	
	//	[1][0]	[1][1]	[1][2]
	//	[2][0]	[2][1]	[2][2]
	//OFFSETS
	//	(X,1,-1)	(X,1,0)		(X,1,1)
	//	(X,0,-1)	(X,0,0)		(X,0,1)
	//	(X,-1,-1)	(X,-1,0)	(X,0,1)

	fXOffsetCenter = -LENGTHOFCUBIESIDE;
	fYOffsetCenter = LENGTHOFCUBIESIDE * (-(row - 1));
	fZOffsetCenter = LENGTHOFCUBIESIDE * (col - 1);

	float fvCol[4];
	float* fvPointer;
	//memcpy(fvCol, CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal), sizeof(CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal)));
	fvPointer = CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal);
	for (int a = 0; a < 3; a++)
	{
		fvCol[a] = fvPointer[a];
	}
	fvCol[3] = m_fAlpha;

	glPushMatrix();
	glTranslatef(fXOffsetCenter, fYOffsetCenter, fZOffsetCenter);
	
	glNormal3f(-1.0f, 0.0f, 0.0f);
	glColor4fv(fvCol);
	//DRAW THIS FACE
	glBegin(GL_TRIANGLES);
		glVertex3f(-HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);		//top left
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom left

		glVertex3f(-HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(-HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom right
	glEnd();
	//this square was just drawn, so set its value to true
	//to prevent it from tyring to be drawwn again
	m_Squares[currFace][row][col].m_bDrawn = true;
	
	glColor3fv(CColors::GetColor(BLACK));
	//TRACE EACH EDGE OF EACH CUBIE TO
	//MAKE A DISTINCTION BETWEEN EACH PIECE

	glBegin(GL_LINES);
		glVertex3f(-HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
	glEnd();
	
	glPopMatrix();
}

void CRubiksCube::DrawRight(int currFace, int row, int col)
{
	//if we have drawn this square already, move to
	//the next square
	if(m_Squares[currFace][row][col].m_bDrawn)
		return;
	float fXOffsetCenter, fYOffsetCenter, fZOffsetCenter;
	//FOR THE RIGHT FACE, WE WANT TO DRAW FROM THE
	//CENTER...SQUARE VALUES
	//	[0][0]	[0][1]	[0][2]	
	//	[1][0]	[1][1]	[1][2]
	//	[2][0]	[2][1]	[2][2]
	//OFFSETS
	//	(X,1,1)	(X,1,0)		(X,1,-1)
	//	(X,0,1)	(X,0,0)		(X,0,-1)
	//	(X,-1,1)(X,-1,0)	(X,0,-1)

	fXOffsetCenter = LENGTHOFCUBIESIDE;
	fYOffsetCenter = LENGTHOFCUBIESIDE * (-(row - 1));
	fZOffsetCenter = LENGTHOFCUBIESIDE * (-(col - 1));

	float fvCol[4];
	float* fvPointer;
	//memcpy(fvCol, CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal), sizeof(CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal)));
	fvPointer = CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal);
	for (int a = 0; a < 3; a++)
	{
		fvCol[a] = fvPointer[a];
	}
	fvCol[3] = m_fAlpha;

	glPushMatrix();
	glTranslatef(fXOffsetCenter, fYOffsetCenter, fZOffsetCenter);
	
	glNormal3f(1.0f, 0.0f, 0.0f);
	glColor4fv(fvCol);
	//DRAW THIS FACE
	glBegin(GL_TRIANGLES);
		glVertex3f(HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom left	
		glVertex3f(HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);		//top left
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top right

		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom right
		glVertex3f(HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);		//top right
	glEnd();
	//this square was just drawn, so set its value to true
	//to prevent it from tyring to be drawwn again
	m_Squares[currFace][row][col].m_bDrawn = true;

	glColor3fv(CColors::GetColor(BLACK));
	//TRACE EACH EDGE OF EACH CUBIE TO
	//MAKE A DISTINCTION BETWEEN EACH PIECE

	glBegin(GL_LINES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
	glEnd();
	
	glPopMatrix();
}

void CRubiksCube::DrawBack(int currFace, int row, int col)
{
	//if we have drawn this square already, move to
	//the next square
	if(m_Squares[currFace][row][col].m_bDrawn)
		return;
	float fXOffsetCenter, fYOffsetCenter, fZOffsetCenter;
	//FOR THE FRONT FACE, WE WANT TO DRAW FROM THE
	//CENTER...SQUARE VALUES
	//	[0][0]	[0][1]	[0][2]	
	//	[1][0]	[1][1]	[1][2]
	//	[2][0]	[2][1]	[2][2]
	//OFFSETS
	//	(-1,1,Z)	(0,1,Z)		(1,1,Z)
	//	(-1,0,Z)	(0,0,Z)		(1,0,Z)
	//	(-1,-1,Z)	(0,-1,Z)	(1,0,Z)

	fXOffsetCenter = LENGTHOFCUBIESIDE * (-(col - 1));
	fYOffsetCenter = LENGTHOFCUBIESIDE * (-(row - 1));
	fZOffsetCenter = -LENGTHOFCUBIESIDE;

	float fvCol[4];
	float* fvPointer;
	//memcpy(fvCol, CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal), sizeof(CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal)));
	fvPointer = CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal);
	for (int a = 0; a < 3; a++)
	{
		fvCol[a] = fvPointer[a];
	}
	fvCol[3] = m_fAlpha;

	glPushMatrix();
	glTranslatef(fXOffsetCenter, fYOffsetCenter, fZOffsetCenter);
	glNormal3f(0.0f, 0.0f, -1.0f);
	glColor4fv(fvCol);
	//DRAW THIS FACE
	glBegin(GL_TRIANGLES);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);		//top left
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);		//top right
		
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);		//top right
	glEnd();

	//this square was just drawn, so set its value to true
	//to prevent it from tyring to be drawwn again
	m_Squares[currFace][row][col].m_bDrawn = true;

	glColor3fv(CColors::GetColor(BLACK));
	//TRACE EACH EDGE OF EACH CUBIE TO
	//MAKE A DISTINCTION BETWEEN EACH PIECE

	glBegin(GL_LINES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
	glEnd();

	
	glPopMatrix();
}

void CRubiksCube::DrawTop(int currFace, int row, int col)
{
	//if we have drawn this square already, move to
	//the next square
	if(m_Squares[currFace][row][col].m_bDrawn)
		return;
	float fXOffsetCenter, fYOffsetCenter, fZOffsetCenter;
	//FOR THE TOP FACE, WE WANT TO DRAW FROM THE
	//CENTER...SQUARE VALUES
	//	[0][0]	[0][1]	[0][2]	
	//	[1][0]	[1][1]	[1][2]
	//	[2][0]	[2][1]	[2][2]
	//OFFSETS
	//	(-1,Y,-1)	(0,Y,-1)	(1,Y,-1)
	//	(-1,Y,0)	(0,Y,0)		(1,Y,0)
	//	(-1,Y,1)	(0,Y,1)		(1,Y,1)

	fXOffsetCenter = LENGTHOFCUBIESIDE * (col - 1);
	fYOffsetCenter = LENGTHOFCUBIESIDE;
	fZOffsetCenter = LENGTHOFCUBIESIDE * (row - 1);

	float fvCol[4];
	float* fvPointer;
	//memcpy(fvCol, CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal), sizeof(CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal)));
	fvPointer = CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal);
	for (int a = 0; a < 3; a++)
	{
		fvCol[a] = fvPointer[a];
	}
	fvCol[3] = m_fAlpha;

	glPushMatrix();
	glTranslatef(fXOffsetCenter, fYOffsetCenter, fZOffsetCenter);
	
	glNormal3f(0.0f, 1.0f, 0.0f);
	glColor4fv(fvCol);
	//DRAW THIS FACE
	glBegin(GL_TRIANGLES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//top right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//top left
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom left
		
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);	//top right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom right
	glEnd();

	//this square was just drawn, so set its value to true
	//to prevent it from tyring to be drawwn again
	m_Squares[currFace][row][col].m_bDrawn = true;

	glColor3fv(CColors::GetColor(BLACK));
	//TRACE EACH EDGE OF EACH CUBIE TO
	//MAKE A DISTINCTION BETWEEN EACH PIECE

	glBegin(GL_LINES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
	glEnd();

	
	glPopMatrix();
}

void CRubiksCube::DrawBottom(int currFace, int row, int col)
{
	//if we have drawn this square already, move to
	//the next square
	if(m_Squares[currFace][row][col].m_bDrawn)
		return;
	float fXOffsetCenter, fYOffsetCenter, fZOffsetCenter;
	//FOR THE TOP FACE, WE WANT TO DRAW FROM THE
	//CENTER...SQUARE VALUES
	//	[0][0]	[0][1]	[0][2]	
	//	[1][0]	[1][1]	[1][2]
	//	[2][0]	[2][1]	[2][2]
	//OFFSETS
	//	(-1,Y,-1)	(0,Y,-1)	(1,Y,-1)
	//	(-1,Y,0)	(0,Y,0)		(1,Y,0)
	//	(-1,Y,1)	(0,Y,1)		(1,Y,1)

	fXOffsetCenter = LENGTHOFCUBIESIDE * (col - 1);
	fYOffsetCenter = -LENGTHOFCUBIESIDE;
	fZOffsetCenter = LENGTHOFCUBIESIDE * (-(row - 1));

	float fvCol[4];
	float* fvPointer;
	//memcpy(fvCol, CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal), sizeof(CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal)));
	fvPointer = CColors::GetColor(m_Squares[currFace][row][col].m_nColorVal);
	for (int a = 0; a < 3; a++)
	{
		fvCol[a] = fvPointer[a];
	}
	fvCol[3] = m_fAlpha;

	glPushMatrix();
	glTranslatef(fXOffsetCenter, fYOffsetCenter, fZOffsetCenter);
	
	glNormal3f(0.0f, -1.0f, 0.0f);
	glColor4fv(fvCol);
	//DRAW THIS FACE
	glBegin(GL_TRIANGLES);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//top left
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//top right
		
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom right
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);		//bottom left
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE);		//top right
	glEnd();

	//this square was just drawn, so set its value to true
	//to prevent it from tyring to be drawwn again
	m_Squares[currFace][row][col].m_bDrawn = true;

	glColor3fv(CColors::GetColor(BLACK));
	//TRACE EACH EDGE OF EACH CUBIE TO
	//MAKE A DISTINCTION BETWEEN EACH PIECE

	glBegin(GL_LINES);
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);

		glVertex3f(-HALFLENGTHOFCUBIESIDE,-HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE,HALFLENGTHOFCUBIESIDE);

		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE, HALFLENGTHOFCUBIESIDE);
		glVertex3f(HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE, -HALFLENGTHOFCUBIESIDE);
	glEnd();

	glPopMatrix();
}

void CRubiksCube::ResetDrawnStatus()
{
	for (int currFace = FRONT; currFace <= BOTTOM; currFace++)	//loop through all the faces
	{
		for (int x = 0; x < DIMENSIONS; x++)
		{
			for (int y = 0; y < DIMENSIONS; y++)
			{
				m_Squares[currFace][x][y].m_bDrawn = false;
			}
		}
	}
}

int	CRubiksCube::GetPieceCoords(int Colors, int* Face, int* Row, int* Col)
{
	int nNumOfSides = 0;	//will be 2 for edge pieces, 3 for corners, 1 for center
	for (int currFace = FRONT; currFace <= BOTTOM; currFace++)
	{
		for (int row = 0; row < DIMENSIONS; row++)
		{
			for (int col = 0; col < DIMENSIONS; col++)
			{
				if (m_Squares[currFace][row][col].m_nColorsSharedBySquare == Colors)
				{
					Face[nNumOfSides]	=	currFace;
					Row[nNumOfSides]	=	row;
					Col[nNumOfSides]	=	col;
					nNumOfSides++;
				}
			}
		}
	}
	return nNumOfSides; //return the number of sides we requested
						//our arrays are filled out with all orientations
						//of the requested piece
}

void CRubiksCube::RotateFront(int currFace, bool bClockwise)
{
	CSquare UpperLeft	= m_Squares[currFace][TOPROW][LEFTCOL];
	CSquare UpperRight	= m_Squares[currFace][TOPROW][RIGHTCOL];
	CSquare LowerRight	= m_Squares[currFace][BOTTOMROW][RIGHTCOL];
	CSquare LowerLeft	= m_Squares[currFace][BOTTOMROW][LEFTCOL];

	CSquare TopEdge		= m_Squares[currFace][TOPROW][MIDDLECOL];
	CSquare	BottomEdge	= m_Squares[currFace][BOTTOMROW][MIDDLECOL];
	CSquare	RightEdge	= m_Squares[currFace][MIDDLEROW][RIGHTCOL];
	CSquare	LeftEdge	= m_Squares[currFace][MIDDLEROW][LEFTCOL];

	CSquare	TopFaceLeftCorner	=	m_Squares[TOP][BOTTOMROW][LEFTCOL];
	CSquare TopFaceEdge			=	m_Squares[TOP][BOTTOMROW][MIDDLECOL];
	CSquare TopFaceRightCorner	=	m_Squares[TOP][BOTTOMROW][RIGHTCOL];

	CSquare RightFaceTopCorner	=	m_Squares[RIGHT][TOPROW][LEFTCOL];
	CSquare RightFaceEdge		=	m_Squares[RIGHT][MIDDLEROW][LEFTCOL];
	CSquare RightFaceBottomCorner=	m_Squares[RIGHT][BOTTOMROW][LEFTCOL];

	CSquare BottomFaceLeftCorner=	m_Squares[BOTTOM][TOPROW][LEFTCOL];
	CSquare BottomFaceEdge		=	m_Squares[BOTTOM][TOPROW][MIDDLECOL];
	CSquare BottomFaceRightCorner=	m_Squares[BOTTOM][TOPROW][RIGHTCOL];
	
	CSquare LeftFaceTopCorner	=	m_Squares[LEFT][TOPROW][RIGHTCOL];
	CSquare LeftFaceEdge		=	m_Squares[LEFT][MIDDLEROW][RIGHTCOL];
	CSquare LeftFaceBottomCorner=	m_Squares[LEFT][BOTTOMROW][RIGHTCOL];
	
	if(bClockwise)
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	BottomEdge;

		m_Squares[TOP][BOTTOMROW][LEFTCOL]		=	LeftFaceBottomCorner;
		m_Squares[TOP][BOTTOMROW][MIDDLECOL]	=	LeftFaceEdge;
		m_Squares[TOP][BOTTOMROW][RIGHTCOL]		=	LeftFaceTopCorner;

		m_Squares[RIGHT][TOPROW][LEFTCOL]		=	TopFaceLeftCorner;
		m_Squares[RIGHT][MIDDLEROW][LEFTCOL]	=	TopFaceEdge;
		m_Squares[RIGHT][BOTTOMROW][LEFTCOL]	=	TopFaceRightCorner;
		
		m_Squares[BOTTOM][TOPROW][LEFTCOL]		=	RightFaceBottomCorner;
		m_Squares[BOTTOM][TOPROW][MIDDLECOL]	=	RightFaceEdge;
		m_Squares[BOTTOM][TOPROW][RIGHTCOL]		=	RightFaceTopCorner;
		
		m_Squares[LEFT][TOPROW][RIGHTCOL]		=	BottomFaceLeftCorner;
		m_Squares[LEFT][MIDDLEROW][RIGHTCOL]	=	BottomFaceEdge;
		m_Squares[LEFT][BOTTOMROW][RIGHTCOL]	=	BottomFaceRightCorner;
	}
	else
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	UpperRight;//LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	LowerRight;//UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	LowerLeft;//UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	UpperLeft;//LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	RightEdge;//LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	LeftEdge;//RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	BottomEdge;//TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	TopEdge;//BottomEdge;

		m_Squares[TOP][BOTTOMROW][LEFTCOL]		=	RightFaceTopCorner;//LeftFaceBottomCorner;
		m_Squares[TOP][BOTTOMROW][MIDDLECOL]	=	RightFaceEdge;//LeftFaceEdge;
		m_Squares[TOP][BOTTOMROW][RIGHTCOL]		=	RightFaceBottomCorner;//LeftFaceTopCorner;

		m_Squares[RIGHT][TOPROW][LEFTCOL]		=	BottomFaceRightCorner;//TopFaceLeftCorner;
		m_Squares[RIGHT][MIDDLEROW][LEFTCOL]	=	BottomFaceEdge;//TopFaceEdge;
		m_Squares[RIGHT][BOTTOMROW][LEFTCOL]	=	BottomFaceLeftCorner;//TopFaceRightCorner;
		
		m_Squares[BOTTOM][TOPROW][LEFTCOL]		=	LeftFaceTopCorner;//RightFaceBottomCorner;
		m_Squares[BOTTOM][TOPROW][MIDDLECOL]	=	LeftFaceEdge;//RightFaceEdge;
		m_Squares[BOTTOM][TOPROW][RIGHTCOL]		=	LeftFaceBottomCorner;//RightFaceTopCorner;
		
		m_Squares[LEFT][TOPROW][RIGHTCOL]		=	TopFaceRightCorner;//BottomFaceLeftCorner;
		m_Squares[LEFT][MIDDLEROW][RIGHTCOL]	=	TopFaceEdge;//BottomFaceEdge;
		m_Squares[LEFT][BOTTOMROW][RIGHTCOL]	=	TopFaceLeftCorner;//BottomFaceRightCorner;
	}
}

void CRubiksCube::RotateLeft(int currFace, bool bClockwise)
{
	CSquare UpperLeft	= m_Squares[currFace][TOPROW][LEFTCOL];
	CSquare UpperRight	= m_Squares[currFace][TOPROW][RIGHTCOL];
	CSquare LowerRight	= m_Squares[currFace][BOTTOMROW][RIGHTCOL];
	CSquare LowerLeft	= m_Squares[currFace][BOTTOMROW][LEFTCOL];

	CSquare TopEdge		= m_Squares[currFace][TOPROW][MIDDLECOL];
	CSquare	BottomEdge	= m_Squares[currFace][BOTTOMROW][MIDDLECOL];
	CSquare	RightEdge	= m_Squares[currFace][MIDDLEROW][RIGHTCOL];
	CSquare	LeftEdge	= m_Squares[currFace][MIDDLEROW][LEFTCOL];

	CSquare	TopFaceLeftCorner	=	m_Squares[TOP][TOPROW][LEFTCOL];
	CSquare TopFaceEdge			=	m_Squares[TOP][MIDDLEROW][LEFTCOL];
	CSquare TopFaceRightCorner	=	m_Squares[TOP][BOTTOMROW][LEFTCOL];

	CSquare RightFaceTopCorner	=	m_Squares[FRONT][TOPROW][LEFTCOL];
	CSquare RightFaceEdge		=	m_Squares[FRONT][MIDDLEROW][LEFTCOL];
	CSquare RightFaceBottomCorner=	m_Squares[FRONT][BOTTOMROW][LEFTCOL];

	CSquare BottomFaceLeftCorner=	m_Squares[BOTTOM][BOTTOMROW][LEFTCOL];
	CSquare BottomFaceEdge		=	m_Squares[BOTTOM][MIDDLEROW][LEFTCOL];
	CSquare BottomFaceRightCorner=	m_Squares[BOTTOM][TOPROW][LEFTCOL];
	
	CSquare LeftFaceTopCorner	=	m_Squares[BACK][TOPROW][RIGHTCOL];
	CSquare LeftFaceEdge		=	m_Squares[BACK][MIDDLEROW][RIGHTCOL];
	CSquare LeftFaceBottomCorner=	m_Squares[BACK][BOTTOMROW][RIGHTCOL];

	if(bClockwise)
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	BottomEdge;

		m_Squares[TOP][TOPROW][LEFTCOL]		=	LeftFaceBottomCorner;
		m_Squares[TOP][MIDDLEROW][LEFTCOL]	=	LeftFaceEdge;
		m_Squares[TOP][BOTTOMROW][LEFTCOL]	=	LeftFaceTopCorner;

		m_Squares[FRONT][TOPROW][LEFTCOL]		=	TopFaceLeftCorner;
		m_Squares[FRONT][MIDDLEROW][LEFTCOL]	=	TopFaceEdge;
		m_Squares[FRONT][BOTTOMROW][LEFTCOL]	=	TopFaceRightCorner;
		
		m_Squares[BOTTOM][BOTTOMROW][LEFTCOL]	=	RightFaceBottomCorner;
		m_Squares[BOTTOM][MIDDLEROW][LEFTCOL]	=	RightFaceEdge;
		m_Squares[BOTTOM][TOPROW][LEFTCOL]		=	RightFaceTopCorner;
		
		m_Squares[BACK][TOPROW][RIGHTCOL]		=	BottomFaceLeftCorner;
		m_Squares[BACK][MIDDLEROW][RIGHTCOL]	=	BottomFaceEdge;
		m_Squares[BACK][BOTTOMROW][RIGHTCOL]	=	BottomFaceRightCorner;
	}
	else
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	UpperRight;//LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	LowerRight;//UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	LowerLeft;//UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	UpperLeft;//LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	RightEdge;//LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	LeftEdge;//RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	BottomEdge;//TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	TopEdge;//BottomEdge;

		m_Squares[TOP][TOPROW][LEFTCOL]		=	RightFaceTopCorner;//LeftFaceBottomCorner;
		m_Squares[TOP][MIDDLEROW][LEFTCOL]	=	RightFaceEdge;//LeftFaceEdge;
		m_Squares[TOP][BOTTOMROW][LEFTCOL]	=	RightFaceBottomCorner;//LeftFaceTopCorner;

		m_Squares[FRONT][TOPROW][LEFTCOL]		=	BottomFaceRightCorner;//TopFaceLeftCorner;
		m_Squares[FRONT][MIDDLEROW][LEFTCOL]	=	BottomFaceEdge;//TopFaceEdge;
		m_Squares[FRONT][BOTTOMROW][LEFTCOL]	=	BottomFaceLeftCorner;//TopFaceRightCorner;
		
		m_Squares[BOTTOM][BOTTOMROW][LEFTCOL]	=	LeftFaceTopCorner;//RightFaceBottomCorner;
		m_Squares[BOTTOM][MIDDLEROW][LEFTCOL]	=	LeftFaceEdge;//RightFaceEdge;
		m_Squares[BOTTOM][TOPROW][LEFTCOL]		=	LeftFaceBottomCorner;//RightFaceTopCorner;
		
		m_Squares[BACK][TOPROW][RIGHTCOL]		=	TopFaceRightCorner;//BottomFaceLeftCorner;
		m_Squares[BACK][MIDDLEROW][RIGHTCOL]	=	TopFaceEdge;//BottomFaceEdge;
		m_Squares[BACK][BOTTOMROW][RIGHTCOL]	=	TopFaceLeftCorner;//BottomFaceRightCorner;
	}
}

void CRubiksCube::RotateBack(int currFace, bool bClockwise)
{
	CSquare UpperLeft	= m_Squares[currFace][TOPROW][LEFTCOL];
	CSquare UpperRight	= m_Squares[currFace][TOPROW][RIGHTCOL];
	CSquare LowerRight	= m_Squares[currFace][BOTTOMROW][RIGHTCOL];
	CSquare LowerLeft	= m_Squares[currFace][BOTTOMROW][LEFTCOL];

	CSquare TopEdge		= m_Squares[currFace][TOPROW][MIDDLECOL];
	CSquare	BottomEdge	= m_Squares[currFace][BOTTOMROW][MIDDLECOL];
	CSquare	RightEdge	= m_Squares[currFace][MIDDLEROW][RIGHTCOL];
	CSquare	LeftEdge	= m_Squares[currFace][MIDDLEROW][LEFTCOL];

	CSquare	TopFaceLeftCorner	=	m_Squares[TOP][TOPROW][RIGHTCOL];
	CSquare TopFaceEdge			=	m_Squares[TOP][TOPROW][MIDDLECOL];
	CSquare TopFaceRightCorner	=	m_Squares[TOP][TOPROW][LEFTCOL];

	CSquare RightFaceTopCorner	=	m_Squares[LEFT][TOPROW][LEFTCOL];
	CSquare RightFaceEdge		=	m_Squares[LEFT][MIDDLEROW][LEFTCOL];
	CSquare RightFaceBottomCorner=	m_Squares[LEFT][BOTTOMROW][LEFTCOL];

	CSquare BottomFaceLeftCorner=	m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL];
	CSquare BottomFaceEdge		=	m_Squares[BOTTOM][BOTTOMROW][MIDDLECOL];
	CSquare BottomFaceRightCorner=	m_Squares[BOTTOM][BOTTOMROW][LEFTCOL];
	
	CSquare LeftFaceTopCorner	=	m_Squares[RIGHT][TOPROW][RIGHTCOL];
	CSquare LeftFaceEdge		=	m_Squares[RIGHT][MIDDLEROW][RIGHTCOL];
	CSquare LeftFaceBottomCorner=	m_Squares[RIGHT][BOTTOMROW][RIGHTCOL];

	if(bClockwise)
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	BottomEdge;

		m_Squares[TOP][TOPROW][RIGHTCOL]	=	LeftFaceBottomCorner;
		m_Squares[TOP][TOPROW][MIDDLECOL]	=	LeftFaceEdge;
		m_Squares[TOP][TOPROW][LEFTCOL]		=	LeftFaceTopCorner;

		m_Squares[LEFT][TOPROW][LEFTCOL]	=	TopFaceLeftCorner;
		m_Squares[LEFT][MIDDLEROW][LEFTCOL]	=	TopFaceEdge;
		m_Squares[LEFT][BOTTOMROW][LEFTCOL]	=	TopFaceRightCorner;
		
		m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL]	=	RightFaceBottomCorner;
		m_Squares[BOTTOM][BOTTOMROW][MIDDLECOL]	=	RightFaceEdge;
		m_Squares[BOTTOM][BOTTOMROW][LEFTCOL]	=	RightFaceTopCorner;
		
		m_Squares[RIGHT][TOPROW][RIGHTCOL]		=	BottomFaceLeftCorner;
		m_Squares[RIGHT][MIDDLEROW][RIGHTCOL]	=	BottomFaceEdge;
		m_Squares[RIGHT][BOTTOMROW][RIGHTCOL]	=	BottomFaceRightCorner;
	}
	else
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	UpperRight;//LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	LowerRight;//UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	LowerLeft;//UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	UpperLeft;//LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	RightEdge;//LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	LeftEdge;//RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	BottomEdge;//TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	TopEdge;//BottomEdge;

		m_Squares[TOP][TOPROW][RIGHTCOL]	=	RightFaceTopCorner;//LeftFaceBottomCorner;
		m_Squares[TOP][TOPROW][MIDDLECOL]	=	RightFaceEdge;//LeftFaceEdge;
		m_Squares[TOP][TOPROW][LEFTCOL]		=	RightFaceBottomCorner;//LeftFaceTopCorner;

		m_Squares[LEFT][TOPROW][LEFTCOL]	=	BottomFaceRightCorner;//TopFaceLeftCorner;
		m_Squares[LEFT][MIDDLEROW][LEFTCOL]	=	BottomFaceEdge;//TopFaceEdge;
		m_Squares[LEFT][BOTTOMROW][LEFTCOL]	=	BottomFaceLeftCorner;//TopFaceRightCorner;
		
		m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL]	=	LeftFaceTopCorner;//RightFaceBottomCorner;
		m_Squares[BOTTOM][BOTTOMROW][MIDDLECOL]	=	LeftFaceEdge;//RightFaceEdge;
		m_Squares[BOTTOM][BOTTOMROW][LEFTCOL]	=	LeftFaceBottomCorner;//RightFaceTopCorner;
		
		m_Squares[RIGHT][TOPROW][RIGHTCOL]		=	TopFaceRightCorner;//BottomFaceLeftCorner;
		m_Squares[RIGHT][MIDDLEROW][RIGHTCOL]	=	TopFaceEdge;//BottomFaceEdge;
		m_Squares[RIGHT][BOTTOMROW][RIGHTCOL]	=	TopFaceLeftCorner;//BottomFaceRightCorner;
	}
}

void CRubiksCube::RotateRight(int currFace, bool bClockwise)
{
	CSquare UpperLeft	= m_Squares[currFace][TOPROW][LEFTCOL];
	CSquare UpperRight	= m_Squares[currFace][TOPROW][RIGHTCOL];
	CSquare LowerRight	= m_Squares[currFace][BOTTOMROW][RIGHTCOL];
	CSquare LowerLeft	= m_Squares[currFace][BOTTOMROW][LEFTCOL];

	CSquare TopEdge		= m_Squares[currFace][TOPROW][MIDDLECOL];
	CSquare	BottomEdge	= m_Squares[currFace][BOTTOMROW][MIDDLECOL];
	CSquare	RightEdge	= m_Squares[currFace][MIDDLEROW][RIGHTCOL];
	CSquare	LeftEdge	= m_Squares[currFace][MIDDLEROW][LEFTCOL];

	CSquare	TopFaceLeftCorner	=	m_Squares[TOP][BOTTOMROW][RIGHTCOL];
	CSquare TopFaceEdge			=	m_Squares[TOP][MIDDLEROW][RIGHTCOL];
	CSquare TopFaceRightCorner	=	m_Squares[TOP][TOPROW][RIGHTCOL];

	CSquare RightFaceTopCorner	=	m_Squares[BACK][TOPROW][LEFTCOL];
	CSquare RightFaceEdge		=	m_Squares[BACK][MIDDLEROW][LEFTCOL];
	CSquare RightFaceBottomCorner=	m_Squares[BACK][BOTTOMROW][LEFTCOL];

	CSquare BottomFaceLeftCorner=	m_Squares[BOTTOM][TOPROW][RIGHTCOL];
	CSquare BottomFaceEdge		=	m_Squares[BOTTOM][MIDDLEROW][RIGHTCOL];
	CSquare BottomFaceRightCorner=	m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL];
	
	CSquare LeftFaceTopCorner	=	m_Squares[FRONT][TOPROW][RIGHTCOL];
	CSquare LeftFaceEdge		=	m_Squares[FRONT][MIDDLEROW][RIGHTCOL];
	CSquare LeftFaceBottomCorner=	m_Squares[FRONT][BOTTOMROW][RIGHTCOL];

	if(bClockwise)
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	BottomEdge;

		m_Squares[TOP][BOTTOMROW][RIGHTCOL]	=	LeftFaceBottomCorner;
		m_Squares[TOP][MIDDLEROW][RIGHTCOL]	=	LeftFaceEdge;
		m_Squares[TOP][TOPROW][RIGHTCOL]	=	LeftFaceTopCorner;

		m_Squares[BACK][TOPROW][LEFTCOL]	=	TopFaceLeftCorner;
		m_Squares[BACK][MIDDLEROW][LEFTCOL]	=	TopFaceEdge;
		m_Squares[BACK][BOTTOMROW][LEFTCOL]	=	TopFaceRightCorner;
		
		m_Squares[BOTTOM][TOPROW][RIGHTCOL]	=	RightFaceBottomCorner;
		m_Squares[BOTTOM][MIDDLEROW][RIGHTCOL]	=	RightFaceEdge;
		m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL]	=	RightFaceTopCorner;
		
		m_Squares[FRONT][TOPROW][RIGHTCOL]		=	BottomFaceLeftCorner;
		m_Squares[FRONT][MIDDLEROW][RIGHTCOL]	=	BottomFaceEdge;
		m_Squares[FRONT][BOTTOMROW][RIGHTCOL]	=	BottomFaceRightCorner;
	}
	else
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	UpperRight;//LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	LowerRight;//UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	LowerLeft;//UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	UpperLeft;//LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	RightEdge;//LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	LeftEdge;//RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	BottomEdge;//TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	TopEdge;//BottomEdge;

		m_Squares[TOP][BOTTOMROW][RIGHTCOL]	=	RightFaceTopCorner;//LeftFaceBottomCorner;
		m_Squares[TOP][MIDDLEROW][RIGHTCOL]	=	RightFaceEdge;//LeftFaceEdge;
		m_Squares[TOP][TOPROW][RIGHTCOL]	=	RightFaceBottomCorner;//LeftFaceTopCorner;

		m_Squares[BACK][TOPROW][LEFTCOL]	=	BottomFaceRightCorner;//TopFaceLeftCorner;
		m_Squares[BACK][MIDDLEROW][LEFTCOL]	=	BottomFaceEdge;//TopFaceEdge;
		m_Squares[BACK][BOTTOMROW][LEFTCOL]	=	BottomFaceLeftCorner;//TopFaceRightCorner;
		
		m_Squares[BOTTOM][TOPROW][RIGHTCOL]		=	LeftFaceTopCorner;//RightFaceBottomCorner;
		m_Squares[BOTTOM][MIDDLEROW][RIGHTCOL]		=	LeftFaceEdge;//RightFaceEdge;
		m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL]	=	LeftFaceBottomCorner;//RightFaceTopCorner;
		
		m_Squares[FRONT][TOPROW][RIGHTCOL]		=	TopFaceRightCorner;//BottomFaceLeftCorner;
		m_Squares[FRONT][MIDDLEROW][RIGHTCOL]	=	TopFaceEdge;//BottomFaceEdge;
		m_Squares[FRONT][BOTTOMROW][RIGHTCOL]	=	TopFaceLeftCorner;//BottomFaceRightCorner;
	}
}

void CRubiksCube::RotateBottom(int currFace, bool bClockwise)
{
	CSquare UpperLeft	= m_Squares[currFace][TOPROW][LEFTCOL];
	CSquare UpperRight	= m_Squares[currFace][TOPROW][RIGHTCOL];
	CSquare LowerRight	= m_Squares[currFace][BOTTOMROW][RIGHTCOL];
	CSquare LowerLeft	= m_Squares[currFace][BOTTOMROW][LEFTCOL];

	CSquare TopEdge		= m_Squares[currFace][TOPROW][MIDDLECOL];
	CSquare	BottomEdge	= m_Squares[currFace][BOTTOMROW][MIDDLECOL];
	CSquare	RightEdge	= m_Squares[currFace][MIDDLEROW][RIGHTCOL];
	CSquare	LeftEdge	= m_Squares[currFace][MIDDLEROW][LEFTCOL];

	CSquare	TopFaceLeftCorner	=	m_Squares[FRONT][BOTTOMROW][LEFTCOL];
	CSquare TopFaceEdge			=	m_Squares[FRONT][BOTTOMROW][MIDDLECOL];
	CSquare TopFaceRightCorner	=	m_Squares[FRONT][BOTTOMROW][RIGHTCOL];

	CSquare RightFaceTopCorner	=	m_Squares[RIGHT][BOTTOMROW][LEFTCOL];
	CSquare RightFaceEdge		=	m_Squares[RIGHT][BOTTOMROW][MIDDLECOL];
	CSquare RightFaceBottomCorner=	m_Squares[RIGHT][BOTTOMROW][RIGHTCOL];

	CSquare BottomFaceLeftCorner=	m_Squares[BACK][BOTTOMROW][RIGHTCOL];
	CSquare BottomFaceEdge		=	m_Squares[BACK][BOTTOMROW][MIDDLECOL];
	CSquare BottomFaceRightCorner=	m_Squares[BACK][BOTTOMROW][LEFTCOL];
	
	CSquare LeftFaceTopCorner	=	m_Squares[LEFT][BOTTOMROW][RIGHTCOL];
	CSquare LeftFaceEdge		=	m_Squares[LEFT][BOTTOMROW][MIDDLECOL];
	CSquare LeftFaceBottomCorner=	m_Squares[LEFT][BOTTOMROW][LEFTCOL];

	if(bClockwise)
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	BottomEdge;

		m_Squares[FRONT][BOTTOMROW][LEFTCOL]	=	LeftFaceBottomCorner;
		m_Squares[FRONT][BOTTOMROW][MIDDLECOL]	=	LeftFaceEdge;
		m_Squares[FRONT][BOTTOMROW][RIGHTCOL]	=	LeftFaceTopCorner;

		m_Squares[RIGHT][BOTTOMROW][LEFTCOL]	=	TopFaceLeftCorner;
		m_Squares[RIGHT][BOTTOMROW][MIDDLECOL]	=	TopFaceEdge;
		m_Squares[RIGHT][BOTTOMROW][RIGHTCOL]	=	TopFaceRightCorner;
		
		m_Squares[BACK][BOTTOMROW][RIGHTCOL]	=	RightFaceBottomCorner;
		m_Squares[BACK][BOTTOMROW][MIDDLECOL]	=	RightFaceEdge;
		m_Squares[BACK][BOTTOMROW][LEFTCOL]		=	RightFaceTopCorner;
		
		m_Squares[LEFT][BOTTOMROW][RIGHTCOL]	=	BottomFaceLeftCorner;
		m_Squares[LEFT][BOTTOMROW][MIDDLECOL]	=	BottomFaceEdge;
		m_Squares[LEFT][BOTTOMROW][LEFTCOL]		=	BottomFaceRightCorner;
	}
	else
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	UpperRight;//LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	LowerRight;//UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	LowerLeft;//UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	UpperLeft;//LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	RightEdge;//LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	LeftEdge;//RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	BottomEdge;//TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	TopEdge;//BottomEdge;

		m_Squares[FRONT][BOTTOMROW][LEFTCOL]	=	RightFaceTopCorner;//LeftFaceBottomCorner;
		m_Squares[FRONT][BOTTOMROW][MIDDLECOL]	=	RightFaceEdge;//LeftFaceEdge;
		m_Squares[FRONT][BOTTOMROW][RIGHTCOL]	=	RightFaceBottomCorner;//LeftFaceTopCorner;

		m_Squares[RIGHT][BOTTOMROW][LEFTCOL]	=	BottomFaceRightCorner;//TopFaceLeftCorner;
		m_Squares[RIGHT][BOTTOMROW][MIDDLECOL]	=	BottomFaceEdge;//TopFaceEdge;
		m_Squares[RIGHT][BOTTOMROW][RIGHTCOL]	=	BottomFaceLeftCorner;//TopFaceRightCorner;
		
		m_Squares[BACK][BOTTOMROW][RIGHTCOL]	=	LeftFaceTopCorner;//RightFaceBottomCorner;
		m_Squares[BACK][BOTTOMROW][MIDDLECOL]	=	LeftFaceEdge;//RightFaceEdge;
		m_Squares[BACK][BOTTOMROW][LEFTCOL]		=	LeftFaceBottomCorner;//RightFaceTopCorner;
		
		m_Squares[LEFT][BOTTOMROW][RIGHTCOL]	=	TopFaceRightCorner;//BottomFaceLeftCorner;
		m_Squares[LEFT][BOTTOMROW][MIDDLECOL]	=	TopFaceEdge;//BottomFaceEdge;
		m_Squares[LEFT][BOTTOMROW][LEFTCOL]		=	TopFaceLeftCorner;//BottomFaceRightCorner;
	}
}

void CRubiksCube::RotateTop(int currFace, bool bClockwise)
{
	CSquare UpperLeft	= m_Squares[currFace][TOPROW][LEFTCOL];
	CSquare UpperRight	= m_Squares[currFace][TOPROW][RIGHTCOL];
	CSquare LowerRight	= m_Squares[currFace][BOTTOMROW][RIGHTCOL];
	CSquare LowerLeft	= m_Squares[currFace][BOTTOMROW][LEFTCOL];

	CSquare TopEdge		= m_Squares[currFace][TOPROW][MIDDLECOL];
	CSquare	BottomEdge	= m_Squares[currFace][BOTTOMROW][MIDDLECOL];
	CSquare	RightEdge	= m_Squares[currFace][MIDDLEROW][RIGHTCOL];
	CSquare	LeftEdge	= m_Squares[currFace][MIDDLEROW][LEFTCOL];

	CSquare	TopFaceLeftCorner	=	m_Squares[BACK][TOPROW][RIGHTCOL];
	CSquare TopFaceEdge			=	m_Squares[BACK][TOPROW][MIDDLECOL];
	CSquare TopFaceRightCorner	=	m_Squares[BACK][TOPROW][LEFTCOL];

	CSquare RightFaceTopCorner	=	m_Squares[RIGHT][TOPROW][RIGHTCOL];
	CSquare RightFaceEdge		=	m_Squares[RIGHT][TOPROW][MIDDLECOL];
	CSquare RightFaceBottomCorner=	m_Squares[RIGHT][TOPROW][LEFTCOL];

	CSquare BottomFaceLeftCorner=	m_Squares[FRONT][TOPROW][LEFTCOL];
	CSquare BottomFaceEdge		=	m_Squares[FRONT][TOPROW][MIDDLECOL];
	CSquare BottomFaceRightCorner=	m_Squares[FRONT][TOPROW][RIGHTCOL];
	
	CSquare LeftFaceTopCorner	=	m_Squares[LEFT][TOPROW][LEFTCOL];
	CSquare LeftFaceEdge		=	m_Squares[LEFT][TOPROW][MIDDLECOL];
	CSquare LeftFaceBottomCorner=	m_Squares[LEFT][TOPROW][RIGHTCOL];

	if(bClockwise)
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	BottomEdge;

		m_Squares[BACK][TOPROW][RIGHTCOL]	=	LeftFaceBottomCorner;
		m_Squares[BACK][TOPROW][MIDDLECOL]	=	LeftFaceEdge;
		m_Squares[BACK][TOPROW][LEFTCOL]	=	LeftFaceTopCorner;

		m_Squares[RIGHT][TOPROW][RIGHTCOL]	=	TopFaceLeftCorner;
		m_Squares[RIGHT][TOPROW][MIDDLECOL]	=	TopFaceEdge;
		m_Squares[RIGHT][TOPROW][LEFTCOL]	=	TopFaceRightCorner;
		
		m_Squares[FRONT][TOPROW][LEFTCOL]	=	RightFaceBottomCorner;
		m_Squares[FRONT][TOPROW][MIDDLECOL]	=	RightFaceEdge;
		m_Squares[FRONT][TOPROW][RIGHTCOL]	=	RightFaceTopCorner;
		
		m_Squares[LEFT][TOPROW][LEFTCOL]	=	BottomFaceLeftCorner;
		m_Squares[LEFT][TOPROW][MIDDLECOL]	=	BottomFaceEdge;
		m_Squares[LEFT][TOPROW][RIGHTCOL]	=	BottomFaceRightCorner;
	}
	else
	{
		m_Squares[currFace][TOPROW][LEFTCOL]	=	UpperRight;//LowerLeft;
		m_Squares[currFace][TOPROW][RIGHTCOL]	=	LowerRight;//UpperLeft;
		m_Squares[currFace][BOTTOMROW][RIGHTCOL]=	LowerLeft;//UpperRight;
		m_Squares[currFace][BOTTOMROW][LEFTCOL]	=	UpperLeft;//LowerRight;

		m_Squares[currFace][TOPROW][MIDDLECOL]	=	RightEdge;//LeftEdge;
		m_Squares[currFace][BOTTOMROW][MIDDLECOL]=	LeftEdge;//RightEdge;
		m_Squares[currFace][MIDDLEROW][RIGHTCOL]=	BottomEdge;//TopEdge;
		m_Squares[currFace][MIDDLEROW][LEFTCOL]	=	TopEdge;//BottomEdge;

		m_Squares[BACK][TOPROW][RIGHTCOL]	=	RightFaceTopCorner;//LeftFaceBottomCorner;
		m_Squares[BACK][TOPROW][MIDDLECOL]	=	RightFaceEdge;//LeftFaceEdge;
		m_Squares[BACK][TOPROW][LEFTCOL]	=	RightFaceBottomCorner;//LeftFaceTopCorner;

		m_Squares[RIGHT][TOPROW][RIGHTCOL]	=	BottomFaceRightCorner;//TopFaceLeftCorner;
		m_Squares[RIGHT][TOPROW][MIDDLECOL]	=	BottomFaceEdge;//TopFaceEdge;
		m_Squares[RIGHT][TOPROW][LEFTCOL]	=	BottomFaceLeftCorner;//TopFaceRightCorner;
		
		m_Squares[FRONT][TOPROW][LEFTCOL]	=	LeftFaceTopCorner;//RightFaceBottomCorner;
		m_Squares[FRONT][TOPROW][MIDDLECOL]	=	LeftFaceEdge;//RightFaceEdge;
		m_Squares[FRONT][TOPROW][RIGHTCOL]	=	LeftFaceBottomCorner;//RightFaceTopCorner;
		
		m_Squares[LEFT][TOPROW][LEFTCOL]	=	TopFaceRightCorner;//BottomFaceLeftCorner;
		m_Squares[LEFT][TOPROW][MIDDLECOL]	=	TopFaceEdge;//BottomFaceEdge;
		m_Squares[LEFT][TOPROW][RIGHTCOL]	=	TopFaceLeftCorner;//BottomFaceRightCorner;
	}
}

bool CRubiksCube::CubeIsSolved()
{
	for (int currFace = FRONT; currFace <= BOTTOM; currFace++)
	{
		for (int row = 0; row < DIMENSIONS; row++)
		{
			for (int col = 0; col < DIMENSIONS; col++)
			{
				//we just found a square that does not match
				//the solved square, so return false
				if (m_Squares[currFace][row][col].m_nColorVal != m_Squares[currFace][MIDDLEROW][MIDDLECOL].m_nColorVal)
				{
					return false;
				}
			}
		}
	}
	//We checked all squares and they all matched
	return true;
}

void CRubiksCube::RotateFace(int Face, bool Direction)
{
	switch(Face)
	{
	case FRONT:
		RotateFront(Face, Direction);
		break;

	case BACK:
		RotateBack(Face, Direction);
		break;

	case RIGHT:
		RotateRight(Face, Direction);
		break;

	case LEFT:
		RotateLeft(Face, Direction);
		break;

	case TOP:
		RotateTop(Face, Direction);
		break;

	case BOTTOM:
		RotateBottom(Face, Direction);
		break;
	}
}

void CRubiksCube::Update()
{
	if (m_bAutoSolve || m_nCubeState == ROTATINGFACE)
	{
		if(m_ListIterator == NULL)
		{
			m_ListIterator = m_ListOfMoves.begin();
			if(m_ListIterator != m_ListOfMoves.end())
			{
				m_nRotatingFace	=	m_ListIterator -> m_nFaceToMove;
				m_bClockwise	=	m_ListIterator -> m_bClockwise;
			}
		}
		//if no face is rotating
		if(m_nRotatingFace == -1)
		{
			//no moves to make, so return  (avoids crashing on the pop_front)
			if (m_ListOfMoves.empty())
			{
				if (CubeIsSolved())
				{
					m_nCubeState = SOLVED;
				}
				return;
			}
			//this allows us to check the size of the list
			//IE, how many more moves there are until solved.
			//if we use an iterator to "walk" the list, this number
			//wouldnt automatically decrement, and we would need
			//a separate varible to keep track of how many moves
			//are left on the list

			//takes a move off the list
			m_ListOfMoves.pop_front();

			//point to the new current move
			m_ListIterator = m_ListOfMoves.begin();

			//if we are not yet at the end, set the next move
			if(m_ListIterator != m_ListOfMoves.end())
			{
				m_nRotatingFace	=	m_ListIterator -> m_nFaceToMove;
				m_bClockwise	=	m_ListIterator -> m_bClockwise;
			}
			else
			{
				m_ListOfMoves.clear();
				m_ListIterator = NULL;
				//go back to the state we were in before
				//making the rotations
				if (m_nPrevCubeState == -1)
				{
					m_nCubeState  = MAKEBOTTOMCROSS;
				}
				else
				{
					m_nCubeState  = m_nPrevCubeState;
				}
			}
		}
	}

	if (CubeIsSolved())
	{
		m_nCubeState = SOLVED;
	}
}

#ifdef _DEBUG
void CRubiksCube::OutputCube()
{
	FILE* Outputfile = fopen("c:\\windows\\desktop\\Cube.txt","w");
	if (!Outputfile)
		return;

	char* szSides[6] = {"Front", "Right", "Top", "Back", "Left", "Bottom"};
	char* szColors[7]= {"White", "Green", "Red", "Yellow","Blue", "Orange", "Black"};
	char szFileOutput[1024];

	for (int currFace = FRONT; currFace <= BOTTOM; currFace++)
	{
		memset(szFileOutput,'\0',sizeof(szFileOutput));
		strcpy(szFileOutput, szSides[currFace]);
		strcat(szFileOutput, "\n**********\n");
		for (int row = 0; row < DIMENSIONS; row++)
		{
			for (int col = 0; col < DIMENSIONS; col++)
			{
				strcat(szFileOutput, "\t");
				strcat(szFileOutput, szColors[m_Squares[currFace][row][col].m_nColorVal]);
			}
			strcat(szFileOutput,"\n");
		}
		fprintf(Outputfile,"%s\n",szFileOutput);
	}

	fclose(Outputfile);
}
#endif

int CRubiksCube::GetNextMoveMakingBottomCross()
{
	int LocalLeft;
	int LocalRight;
	int LocalBack;
	switch(m_nFaceToSolve)
	{
	case FRONT:
		LocalLeft = LEFT;
		LocalRight = RIGHT;
		LocalBack = BACK;
		break;
	case BACK:
		LocalLeft = RIGHT;
		LocalRight = LEFT;
		LocalBack = FRONT;
		break;
	case RIGHT:
		LocalLeft = FRONT;
		LocalRight = BACK;
		LocalBack = LEFT;
		break;
	case LEFT:
		LocalLeft = BACK;
		LocalRight = FRONT;
		LocalBack = RIGHT;
		break;
	case BOTTOM:
	case TOP:
		m_nFaceToSolve++;
		return m_nFaceToSolve;
		break;
	default:
		return (m_nFaceToSolve = 0);//return -1;
	}

	int ColorsToFind = GetSharedColorValueOfFace(m_nFaceToSolve) | GetSharedColorValueOfFace(BOTTOM);
	int FaceSquareIsOn[3];
	int RowSquareIsOn[3];
	int ColSquareIsOn[3];
	int NumOfFacesTouched;
	int FaceWithColorToSolveOnIt, OtherFaceWithColorToSolveOnIt;
	NumOfFacesTouched = GetPieceCoords(ColorsToFind, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn);
	bool bSquareIsOnFaceWeWant = false;
	bool bSquareIsInCorrectOrientation = false;
	bool bSquareInTopLayer = false;
	bool bSquareInMiddleLayer = false;
	bool bSquareInBottomLayer = false;

	bSquareIsOnFaceWeWant = bSquareIsInCorrectOrientation = bSquareInTopLayer = bSquareInMiddleLayer = bSquareInBottomLayer = false;
	
	if(SquareIsInCorrectPosition(NumOfFacesTouched, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn))
	{
		m_nFaceToSolve++;
		if(m_nFaceToSolve >= BOTTOM)
		{
//before we retun step finsihed, lets make sure the cube is at the state we want
			int nCorrectFrontBottom	=	GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(BOTTOM);
			int nCorrectRightBottom	=	GetSharedColorValueOfFace(RIGHT) | GetSharedColorValueOfFace(BOTTOM);;
			int nCorrectLeftBottom	=	GetSharedColorValueOfFace(LEFT)  | GetSharedColorValueOfFace(BOTTOM);;
			int nCorrectBackBottom	=	GetSharedColorValueOfFace(BACK)  | GetSharedColorValueOfFace(BOTTOM);;			
			
			int nCurrentFrontBottom	=	GetSharedColors(FRONT, BOTTOMROW, MIDDLECOL);
			int nCurrentRightBottom	=	GetSharedColors(RIGHT, BOTTOMROW, MIDDLECOL);;
			int nCurrentLeftBottom	=	GetSharedColors(LEFT, BOTTOMROW, MIDDLECOL);;
			int nCurrentBackBottom	=	GetSharedColors(BACK, BOTTOMROW, MIDDLECOL);;

			int nCurrColFront		=	m_Squares[BOTTOM][TOPROW][MIDDLECOL].m_nColorVal;
			int nCurrColRight		=	m_Squares[BOTTOM][MIDDLEROW][RIGHTCOL].m_nColorVal;
			int nCurrColLeft		=	m_Squares[BOTTOM][MIDDLEROW][LEFTCOL].m_nColorVal;
			int nCurrColBack		=	m_Squares[BOTTOM][BOTTOMROW][MIDDLECOL].m_nColorVal;
			int nBottomCol			=	m_Squares[BOTTOM][MIDDLEROW][MIDDLECOL].m_nColorVal;

			if (nCorrectFrontBottom == nCurrentFrontBottom && 
				nCorrectRightBottom == nCurrentRightBottom && 
				nCorrectLeftBottom	== nCurrentLeftBottom &&
				nCorrectBackBottom	== nCurrentBackBottom &&
				(nCurrColFront == nBottomCol &&
				 nCurrColRight == nBottomCol &&
				 nCurrColLeft == nBottomCol &&
				 nCurrColBack == nBottomCol)
				)
			{
				return STEPFINISHED;
			}
			else
			{
				m_nFaceToSolve = FRONT;
				return m_nFaceToSolve;
			}
		}
		else
		{
			return m_nFaceToSolve;
		}
	}

	for (int x = 0; x < NumOfFacesTouched; x++)
	{
		if(FaceSquareIsOn[x] == LocalLeft || FaceSquareIsOn[x] == LocalRight || FaceSquareIsOn[x] == LocalBack || FaceSquareIsOn[x] == m_nFaceToSolve) 
		{
			if (RowSquareIsOn[x] == TOPROW)
			{
				bSquareInTopLayer = true;
			}
			else if(RowSquareIsOn[x] == BOTTOMROW)
			{
				bSquareInBottomLayer = true;
			}
			else if(RowSquareIsOn[x] == MIDDLEROW)
			{
				bSquareInMiddleLayer = true;
			}
		}
		
		if(m_Squares[FaceSquareIsOn[x]][RowSquareIsOn[x]][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			FaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
		else
		{
			OtherFaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}

		if (FaceSquareIsOn[x] == m_nFaceToSolve)
		{
			bSquareIsOnFaceWeWant = true;
		}

		//if (bSquareIsOnFaceWeWant && bSquareInTopLayer && (m_Squares[m_nFaceToSolve][TOPROW][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal))
		if (bSquareIsOnFaceWeWant && bSquareInTopLayer && (m_Squares[m_nFaceToSolve][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal))
		{
			bSquareIsInCorrectOrientation = true;
		}
	}
///////////////CHECK SQUARE LOCATIONS AND ORIENTATIONS
	if(bSquareInTopLayer && !bSquareIsOnFaceWeWant)
	{
		CMove aMove;
		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove =	TOP;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	if(bSquareInTopLayer && bSquareIsOnFaceWeWant && bSquareIsInCorrectOrientation)
	{
		CMove aMove;
		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove =	m_nFaceToSolve;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	if (bSquareInTopLayer && bSquareIsOnFaceWeWant &&!bSquareIsInCorrectOrientation)
	{
		CMove aMove;
		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove =	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove =	LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove =	m_nFaceToSolve;
		m_ListOfMoves.push_back(aMove);				

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove =	LocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	if (bSquareInMiddleLayer)
	{
		//F'U'F
		bool bLocalClockwise = false;
		if (m_Squares[FaceWithColorToSolveOnIt][MIDDLEROW][LEFTCOL].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			bLocalClockwise = true;
		}
		else
		{
			bLocalClockwise = false;
		}
		
		CMove aMove;
		aMove.m_bClockwise = bLocalClockwise;
		aMove.m_nFaceToMove = FaceWithColorToSolveOnIt;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = bLocalClockwise;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = !bLocalClockwise;
		aMove.m_nFaceToMove = FaceWithColorToSolveOnIt;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	if (bSquareInBottomLayer)
	{
		//F''U'F''
		bool bSquareOnCorrectFace = true;
		int nLocalFaceToRotate = FaceWithColorToSolveOnIt;
		CMove aMove;
		
		if (FaceWithColorToSolveOnIt == BOTTOM)
		{
			bSquareOnCorrectFace = false;
		}

		if (!bSquareOnCorrectFace)
		{
			nLocalFaceToRotate = OtherFaceWithColorToSolveOnIt;
		}

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = nLocalFaceToRotate;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = nLocalFaceToRotate;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	return m_nFaceToSolve;
}

int CRubiksCube::GetNextMoveMakingBottomCorners()
{
	static const int Corners[3] = 
	{GetSharedColorValueOfFace(BOTTOM) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(LEFT),
	 GetSharedColorValueOfFace(BOTTOM) | GetSharedColorValueOfFace(BACK)  | GetSharedColorValueOfFace(LEFT),
	 GetSharedColorValueOfFace(BOTTOM) | GetSharedColorValueOfFace(BACK)  | GetSharedColorValueOfFace(RIGHT)};

	int LocalFront, LocalBack, LocalRight, LocalLeft;

	static int CurrCorner = 0;

	switch (CurrCorner)
	{
	case 0:
		LocalFront = LEFT;
		LocalBack  = RIGHT;
		LocalRight = FRONT;
		LocalLeft  = BACK;
		break;
	case 1:
		LocalFront = BACK;
		LocalBack  = FRONT;
		LocalRight = LEFT;
		LocalLeft  = RIGHT;
		break;
	case 2:
		LocalFront = RIGHT;
		LocalBack  = LEFT;
		LocalRight = BACK;
		LocalLeft  = FRONT;
		break;
	}

	int ColorsToFind = Corners[CurrCorner];
	int FaceSquareIsOn[3];
	int RowSquareIsOn[3];
	int ColSquareIsOn[3];
	int NumOfFacesTouched;
	int FaceWithColorToSolveOnIt, OtherFaceWithColorToSolveOnIt;
	NumOfFacesTouched = GetPieceCoords(ColorsToFind, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn);
	bool bSquareInBottomLayer = false;
	bool bSquareInTopLayer = false;
	bool bSquareInCorrectPosition = false;
	bool bSquareInCorrectOrientation = false;

	if(SquareIsInCorrectPosition(NumOfFacesTouched, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn))
	{
		CurrCorner++;
		if(CurrCorner >= DIMENSIONS)
		{
			CurrCorner = 0;

			int nCurrentBotTopLeft	=	m_Squares[BOTTOM][TOPROW][LEFTCOL].m_nColorsSharedBySquare;
			int nCurrentBotBotLeft	=	m_Squares[BOTTOM][BOTTOMROW][LEFTCOL].m_nColorsSharedBySquare;
			int nCurrentBotBotRight	=	m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL].m_nColorsSharedBySquare;

			int nCorner1	=	m_Squares[BOTTOM][TOPROW][LEFTCOL].m_nColorVal;
			int nCorner2	=	m_Squares[BOTTOM][BOTTOMROW][LEFTCOL].m_nColorVal;
			int nCorner3	=	m_Squares[BOTTOM][BOTTOMROW][RIGHTCOL].m_nColorVal;
			int nBotCol		=	m_Squares[BOTTOM][MIDDLEROW][MIDDLECOL].m_nColorVal;

			if( nCurrentBotTopLeft == Corners[0] &&
				nCurrentBotBotLeft == Corners[1] &&
				nCurrentBotBotRight == Corners[2] &&
				(nCorner1 == nBotCol &&
				 nCorner2 == nBotCol &&
				 nCorner3 == nBotCol)
				)
			{
				return STEPFINISHED;
			}
			else
			{
				return CurrCorner;
			}
		}
		else
		{
			return CurrCorner;
		}
	}

	for (int x = 0; x < DIMENSIONS; x++)
	{
		if (FRONT == FaceSquareIsOn[x] || BACK == FaceSquareIsOn[x] || LEFT == FaceSquareIsOn[x] || RIGHT == FaceSquareIsOn[x])
		{
			if (RowSquareIsOn[x] == TOPROW)
			{
				bSquareInTopLayer = true;
			}
			else if (RowSquareIsOn[x] == BOTTOMROW)
			{
				bSquareInBottomLayer = true;
			}
		}
		//we only need to know ONE of the 3 faces that has
		//the wrong color on it.
		if(m_Squares[FaceSquareIsOn[x]][RowSquareIsOn[x]][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			FaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
		else
		{
			OtherFaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
	}

	//if it is in the bottom layer, and we got here
	//it must be in the wrong position, OR orientation
	if (bSquareInBottomLayer)
	{
		//F'U'F
		int LocalFace = OtherFaceWithColorToSolveOnIt;
		bool bLocalDirection = false;

		if (LocalFace == BOTTOM)
		{
			LocalFace = FaceWithColorToSolveOnIt;
		}
		//if sqaure we are looking at in right col
		if (m_Squares[LocalFace][BOTTOMROW][LEFTCOL].m_nColorsSharedBySquare == Corners[CurrCorner])
		{
			bLocalDirection = true;
		}
		else
		{
			bLocalDirection = false;
		}

		CMove aMove;
		aMove.m_bClockwise = bLocalDirection;
		aMove.m_nFaceToMove = LocalFace;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = bLocalDirection;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = !bLocalDirection;
		aMove.m_nFaceToMove = LocalFace;
		m_ListOfMoves.push_back(aMove);
		
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	//if the square on the front face matches its middle color
	if (bSquareInTopLayer && (m_Squares[LocalFront][TOPROW][RIGHTCOL].m_nColorVal == m_Squares[LocalFront][MIDDLEROW][MIDDLEROW].m_nColorVal && m_Squares[LocalFront][TOPROW][RIGHTCOL].m_nColorsSharedBySquare == Corners[CurrCorner]))
	{
		//RUR'
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}

	if (bSquareInTopLayer && (m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorVal == m_Squares[LocalFront][MIDDLEROW][MIDDLEROW].m_nColorVal && m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorsSharedBySquare == Corners[CurrCorner]))
	{
		//RU'R'U''RUR'
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}

	if (bSquareInTopLayer && (m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorVal == m_Squares[LocalRight][MIDDLEROW][MIDDLECOL].m_nColorVal && m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorsSharedBySquare == Corners[CurrCorner]))
	{
		//F'U'F
		CMove aMove;
		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalFront;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalFront;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}

	// if we get here, it is in the top layer, but totally
	// wrong location, so rotate the top face once
	if (bSquareInTopLayer)
	{
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}
	return CurrCorner;
}

int CRubiksCube::GetNextMoveSolvingEdges()
{
	//the working corner will be defined by
	//the SHAREDCOLORS.  we know where it will start
	//but we dont necessarily know where it will end up
	//but it WILL always end up in the [BOTTOMROW][RIGHTCOL]
	//of the face we are trying to solve...so we can just
	//get its colors from that knowledge
	int LocalFront;
	int LocalLeft;
	int LocalRight;
	int LocalBack;
	static int Edge = 0;
	static const int Edges[3] = 
	{GetSharedColorValueOfFace(LEFT) | GetSharedColorValueOfFace(FRONT),
	 GetSharedColorValueOfFace(BACK) | GetSharedColorValueOfFace(LEFT),
	 GetSharedColorValueOfFace(RIGHT)| GetSharedColorValueOfFace(BACK)};

	switch(m_nFaceToSolve)
	{
	case BACK:
		LocalFront = BACK;
		LocalLeft = RIGHT;
		LocalRight = LEFT;
		LocalBack = FRONT;
		Edge = 1;
		break;
	case RIGHT:
		LocalFront = RIGHT;
		LocalLeft = FRONT;
		LocalRight = BACK;
		LocalBack = LEFT;
		Edge = 2;
		break;
	case LEFT:
		LocalFront = LEFT;
		LocalLeft = BACK;
		LocalRight = FRONT;
		LocalBack = RIGHT;
		Edge = 0;
		break;
	case FRONT:
	case BOTTOM:
	case TOP:
		m_nFaceToSolve++;
		return m_nFaceToSolve;
		break;
	default:
		return (m_nFaceToSolve = 0);//return -1;
	}

	int ColorsToFind = Edges[Edge];
	int FaceSquareIsOn[3];
	int RowSquareIsOn[3];
	int ColSquareIsOn[3];
	int NumOfFacesTouched;
	int FaceWithColorToSolveOnIt, OtherFaceWithColorToSolveOnIt;
	int FaceWhereMiddle;
	NumOfFacesTouched = GetPieceCoords(ColorsToFind, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn);
	bool bSquareInTopLayer = false;
	bool bSquareInMiddleLayer = false;
	bool bSquareInCorrectPosition = false;
	bool bSquareInFaceWeWant = false;
	bool bSquareIsInCorrectOrientation = false;
	static bool bManualOverrideWC = false;
	static int  nFaceOfLastWorkingCorner = FRONT;
	static int nSharedColorsOfWC = 0;

	if(SquareIsInCorrectPosition(NumOfFacesTouched, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn))
	{
		m_nFaceToSolve++;
		if(m_nFaceToSolve >= BOTTOM)
		{
			Edge = 0;
			bManualOverrideWC = false;
			nFaceOfLastWorkingCorner = FRONT;
			nSharedColorsOfWC = 0;
			return STEPFINISHED;
		}
		else
		{
			return m_nFaceToSolve;
		}
	}

	if (0 == nSharedColorsOfWC)
	{
		nSharedColorsOfWC = GetSharedColors(nFaceOfLastWorkingCorner, BOTTOMROW, RIGHTCOL);
	}
	//if the shared colors of the bottom right corner, do not
	//equal the bottom right corner of the face we are trying to solve,
	//turn the bottom face
	int nSharedColorsOfFrontRight = GetSharedColors(m_nFaceToSolve, BOTTOMROW, RIGHTCOL);
	
	if (!bManualOverrideWC && (nSharedColorsOfWC != nSharedColorsOfFrontRight))
	{
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = BOTTOM;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}
	//if we get there, the working corner has been placed
	//correctly!!
	//MUST MAKE SURE THAT WHEN THE FINAL MOVES HAVE BEEN
	//PUSHED ON TO THE LIST, THAT WE RESENT THE VARIABLE nSharedColorsOfWC
	//TO ZERO, SO IT WILL RECALCULATE THE WORKING CORNER
	//SINCE IT MAY CHANGE

	for (int x = 0; x < NumOfFacesTouched; x++)
	{
		if(FaceSquareIsOn[x] == LocalLeft || FaceSquareIsOn[x] == LocalRight || FaceSquareIsOn[x] == LocalBack || FaceSquareIsOn[x] == LocalFront) 
		{
			if (RowSquareIsOn[x] == TOPROW)
			{
				bSquareInTopLayer = true;
			}
			
			if(RowSquareIsOn[x] == MIDDLEROW && ColSquareIsOn[x] == RIGHTCOL)
			{
				bSquareInMiddleLayer = true;
				FaceWhereMiddle = FaceSquareIsOn[x];
			}
		}
		
		if(m_Squares[FaceSquareIsOn[x]][RowSquareIsOn[x]][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			FaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
		else
		{
			OtherFaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}

		if (FaceSquareIsOn[x] == m_nFaceToSolve)
		{
			bSquareInFaceWeWant = true;
		}

		if (bSquareInFaceWeWant && bSquareInTopLayer && (m_Squares[m_nFaceToSolve][TOPROW][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal))
		{
			bSquareIsInCorrectOrientation = true;
		}
	}

	if (bSquareInMiddleLayer)
	{
		//we need to get this into the top layer, but this means moving
		//the working corner underneath the square we are trying to move
		//into the top layer
		if (m_Squares[FaceWhereMiddle][BOTTOMROW][RIGHTCOL].m_nColorsSharedBySquare != nSharedColorsOfWC)
		{
			CMove aMove;
			aMove.m_bClockwise = true;
			aMove.m_nFaceToMove = BOTTOM;
			m_ListOfMoves.push_back(aMove);
			m_nPrevCubeState = m_nCubeState;
			m_nCubeState = ROTATINGFACE;
			bManualOverrideWC = true;
			return m_nFaceToSolve;
		}
		//U'F'UF
		CMove aMove;

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = FaceWhereMiddle;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = FaceWhereMiddle;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		bManualOverrideWC = false;
		nFaceOfLastWorkingCorner = FaceWhereMiddle;
		nSharedColorsOfWC = 0;
		return m_nFaceToSolve;
	}

	if (bSquareInTopLayer)
	{
		if (bSquareInFaceWeWant)
		{
			if (bSquareInCorrectPosition)
			{
				//U'F'UF
				CMove aMove;
				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = LocalFront;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = LocalFront;
				m_ListOfMoves.push_back(aMove);				

				m_nPrevCubeState = m_nCubeState;
				m_nCubeState = ROTATINGFACE;
				nSharedColorsOfWC = 0;
				bManualOverrideWC = false;
				nFaceOfLastWorkingCorner = m_nFaceToSolve;
				return m_nFaceToSolve;
			}
			else	//not in the correct position
			{
				//RU'R'
				CMove aMove;

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = LocalRight;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = LocalRight;
				m_ListOfMoves.push_back(aMove);				

				m_nPrevCubeState = m_nCubeState;
				m_nCubeState = ROTATINGFACE;
				nSharedColorsOfWC = 0;
				bManualOverrideWC = false;
				nFaceOfLastWorkingCorner = m_nFaceToSolve;
				return m_nFaceToSolve;
			}
		}
		else	//not on the face we want
		{
			CMove aMove;
			aMove.m_bClockwise = true;
			aMove.m_nFaceToMove = TOP;
			m_ListOfMoves.push_back(aMove);
			m_nPrevCubeState = m_nCubeState;
			m_nCubeState = ROTATINGFACE;
			nFaceOfLastWorkingCorner = m_nFaceToSolve;
			return m_nFaceToSolve;
		}
	}

	return m_nFaceToSolve;
}

int CRubiksCube::MovingWorkingCorner()
{
	const int nLeftFaceBottomRightCorrect = 
	GetSharedColorValueOfFace(LEFT) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(BOTTOM);
	
	const int nCurrent = m_Squares[LEFT][BOTTOMROW][RIGHTCOL].m_nColorsSharedBySquare;

	if (nLeftFaceBottomRightCorrect != nCurrent)
	{
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = BOTTOM;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}
	else
	{
		return STEPFINISHED;
	}
}

int CRubiksCube::FixWorkingCorner()
{
	static const int Corners[1] = 
	{GetSharedColorValueOfFace(BOTTOM) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(RIGHT)};

	int LocalFront, LocalBack, LocalRight, LocalLeft;

	static int CurrCorner = 0;

	switch (CurrCorner)
	{
	case 0:
		LocalFront = FRONT;
		LocalBack  = BACK;
		LocalRight = RIGHT;
		LocalLeft  = LEFT;
		break;
	}

	int ColorsToFind = Corners[CurrCorner];
	int FaceSquareIsOn[3];
	int RowSquareIsOn[3];
	int ColSquareIsOn[3];
	int NumOfFacesTouched;
	int FaceWithColorToSolveOnIt, OtherFaceWithColorToSolveOnIt;
	NumOfFacesTouched = GetPieceCoords(ColorsToFind, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn);
	bool bSquareInBottomLayer = false;
	bool bSquareInTopLayer = false;
	bool bSquareInCorrectPosition = false;
	bool bSquareInCorrectOrientation = false;

	if(SquareIsInCorrectPosition(NumOfFacesTouched, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn))
	{
		CurrCorner++;
		if(CurrCorner >= 1)
		{
			CurrCorner = 0;
			return STEPFINISHED;
		}
		else
		{
			return CurrCorner;
		}
	}

	for (int x = 0; x < DIMENSIONS; x++)
	{
		if (FRONT == FaceSquareIsOn[x] || BACK == FaceSquareIsOn[x] || LEFT == FaceSquareIsOn[x] || RIGHT == FaceSquareIsOn[x])
		{
			if (RowSquareIsOn[x] == TOPROW)
			{
				bSquareInTopLayer = true;
			}
			else if (RowSquareIsOn[x] == BOTTOMROW)
			{
				bSquareInBottomLayer = true;
			}
		}
		//we only need to know ONE of the 3 faces that has
		//the wrong color on it.
		if(m_Squares[FaceSquareIsOn[x]][RowSquareIsOn[x]][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			FaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
		else
		{
			OtherFaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
	}

	//if it is in the bottom layer, and we got here
	//it must be in the wrong position, OR orientation
	if (bSquareInBottomLayer)
	{
		//F'U'F
		int LocalFace = OtherFaceWithColorToSolveOnIt;
		bool bLocalDirection = false;

		if (LocalFace == BOTTOM)
		{
			LocalFace = FaceWithColorToSolveOnIt;
		}
		//if sqaure we are looking at in right col
		if (m_Squares[LocalFace][BOTTOMROW][LEFTCOL].m_nColorsSharedBySquare == Corners[CurrCorner])
		{
			bLocalDirection = true;
		}
		else
		{
			bLocalDirection = false;
		}

		CMove aMove;
		aMove.m_bClockwise = bLocalDirection;
		aMove.m_nFaceToMove = LocalFace;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = bLocalDirection;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = !bLocalDirection;
		aMove.m_nFaceToMove = LocalFace;
		m_ListOfMoves.push_back(aMove);
		
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	//if the square on the front face matches its middle color
	if (bSquareInTopLayer && (m_Squares[LocalFront][TOPROW][RIGHTCOL].m_nColorVal == m_Squares[LocalFront][MIDDLEROW][MIDDLEROW].m_nColorVal && m_Squares[LocalFront][TOPROW][RIGHTCOL].m_nColorsSharedBySquare == Corners[CurrCorner]))
	{
		//RUR'
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}

	if (bSquareInTopLayer && (m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorVal == m_Squares[LocalFront][MIDDLEROW][MIDDLEROW].m_nColorVal && m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorsSharedBySquare == Corners[CurrCorner]))
	{
		//RU'R'U''RUR'
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}

	if (bSquareInTopLayer && (m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorVal == m_Squares[LocalRight][MIDDLEROW][MIDDLECOL].m_nColorVal && m_Squares[LocalRight][TOPROW][LEFTCOL].m_nColorsSharedBySquare == Corners[CurrCorner]))
	{
		//F'U'F
		CMove aMove;
		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalFront;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalFront;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}

	// if we get here, it is in the top layer, but totally
	// wrong location, so rotate the top face once
	if (bSquareInTopLayer)
	{
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return CurrCorner;
	}
	return CurrCorner;
}

int CRubiksCube::FixWorkingEdge()
{
	//the working corner will be defined by
	//the SHAREDCOLORS.  we know where it will start
	//but we dont necessarily know where it will end up
	//but it WILL always end up in the [BOTTOMROW][RIGHTCOL]
	//of the face we are trying to solve...so we can just
	//get its colors from that knowledge
	int LocalFront;
	int LocalLeft;
	int LocalRight;
	int LocalBack;
	static int Edge = 0;
	static const int Edges[1] = 
	{GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(RIGHT)};

	switch(m_nFaceToSolve)
	{
	case FRONT:
		LocalFront = FRONT;
		LocalLeft = LEFT;
		LocalRight = RIGHT;
		LocalBack = BACK;
		Edge = 0;
		break;
	default:
		return (m_nFaceToSolve = 0);
	}

	int ColorsToFind = Edges[Edge];
	int FaceSquareIsOn[3];
	int RowSquareIsOn[3];
	int ColSquareIsOn[3];
	int NumOfFacesTouched;
	int FaceWithColorToSolveOnIt, OtherFaceWithColorToSolveOnIt;
	NumOfFacesTouched = GetPieceCoords(ColorsToFind, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn);
	bool bSquareInTopLayer = false;
	bool bSquareInMiddleLayer = false;
	bool bSquareInCorrectPosition = false;
	bool bSquareInFaceWeWant = false;
	bool bSquareIsInCorrectOrientation = false;

	if(SquareIsInCorrectPosition(NumOfFacesTouched, FaceSquareIsOn, RowSquareIsOn, ColSquareIsOn))
	{
		m_nFaceToSolve++;
		if(m_nFaceToSolve >= 1)
		{
			Edge = 0;
			return STEPFINISHED;
		}
		else
		{
			return m_nFaceToSolve;
		}
	}

	for (int x = 0; x < NumOfFacesTouched; x++)
	{
		if(FaceSquareIsOn[x] == LocalLeft || FaceSquareIsOn[x] == LocalRight || FaceSquareIsOn[x] == LocalBack || FaceSquareIsOn[x] == LocalFront) 
		{
			if (RowSquareIsOn[x] == TOPROW)
			{
				bSquareInTopLayer = true;
			}			

			if (RowSquareIsOn[x] == MIDDLEROW)
			{
				bSquareInMiddleLayer = true;
			}
		}
		
		if(m_Squares[FaceSquareIsOn[x]][RowSquareIsOn[x]][ColSquareIsOn[x]].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			FaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}
		else
		{
			OtherFaceWithColorToSolveOnIt = FaceSquareIsOn[x];
		}

		if (FaceSquareIsOn[x] == m_nFaceToSolve)
		{
			bSquareInFaceWeWant = true;
		}

		if (bSquareInFaceWeWant && bSquareInTopLayer && (m_Squares[m_nFaceToSolve][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[m_nFaceToSolve][MIDDLEROW][MIDDLECOL].m_nColorVal))
		{
			bSquareIsInCorrectOrientation = true;
		}
	}

	if (bSquareInMiddleLayer)
	{
		//URU'R'U'F'UF
		CMove aMove;
		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = false;
		aMove.m_nFaceToMove = LocalFront;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise = true;
		aMove.m_nFaceToMove = LocalFront;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;
		return m_nFaceToSolve;
	}

	if (bSquareInTopLayer)
	{
		if (bSquareInFaceWeWant)
		{
			if (bSquareIsInCorrectOrientation)
			{
				//URU'R'U'F'UF
				CMove aMove;
				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = LocalRight;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = LocalRight;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = LocalFront;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = LocalFront;
				m_ListOfMoves.push_back(aMove);

				m_nPrevCubeState = m_nCubeState;
				m_nCubeState = ROTATINGFACE;
				return m_nFaceToSolve;
			}
			else	//not in the correct position
			{
				//F'UFURU'R'
				CMove aMove;

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = LocalFront;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = LocalFront;
				m_ListOfMoves.push_back(aMove);				

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);	

				aMove.m_bClockwise = true;
				aMove.m_nFaceToMove = LocalRight;
				m_ListOfMoves.push_back(aMove);	

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = TOP;
				m_ListOfMoves.push_back(aMove);	

				aMove.m_bClockwise = false;
				aMove.m_nFaceToMove = LocalRight;
				m_ListOfMoves.push_back(aMove);	

				m_nPrevCubeState = m_nCubeState;
				m_nCubeState = ROTATINGFACE;
				return m_nFaceToSolve;
			}
		}
		else	//not on the face we want
		{
			CMove aMove;
			aMove.m_bClockwise = true;
			aMove.m_nFaceToMove = TOP;
			m_ListOfMoves.push_back(aMove);
			m_nPrevCubeState = m_nCubeState;
			m_nCubeState = ROTATINGFACE;
			return m_nFaceToSolve;
		}
	}

	return m_nFaceToSolve;
}

int CRubiksCube::MakeTopCross()
{
	bool bFrontEdgeCorrect	=	false;
	bool bRightEdgeCorrect	=	false;
	bool bLeftEdgeCorrect	=	false;
	bool bBackEdgeCorrect	=	false;

	if (m_Squares[TOP][BOTTOMROW][MIDDLECOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
	{
		bFrontEdgeCorrect = true;
	}

	if (m_Squares[TOP][MIDDLEROW][LEFTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
	{
		bLeftEdgeCorrect = true;
	}

	if (m_Squares[TOP][MIDDLEROW][RIGHTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
	{
		bRightEdgeCorrect = true;
	}

	if (m_Squares[TOP][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
	{
		bBackEdgeCorrect = true;
	}

	int nLocalFront	=	FRONT;
	int nLocalBack	=	BACK;
	int nLocalRight	=	RIGHT;
	int nLocalLeft	=	LEFT;

	//opposite edges are both on the correct face
	if ((!bFrontEdgeCorrect && !bBackEdgeCorrect) ||
		(!bRightEdgeCorrect && !bLeftEdgeCorrect))
	{
		if (!bFrontEdgeCorrect)	//FIX FRONT AND BACK
		{
			nLocalFront	=	FRONT;
			nLocalBack	=	BACK;
			nLocalRight	=	RIGHT;
			nLocalLeft	=	LEFT;
		}
		else					//FIX RIGHT AND LEFT
		{
			nLocalFront	=	RIGHT;
			nLocalBack	=	LEFT;
			nLocalRight	=	BACK;
			nLocalLeft	=	FRONT;
		}

		//FRUR'U'F'
		CMove aMove;

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalFront;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalFront;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if ((!bFrontEdgeCorrect && !bRightEdgeCorrect) || 
		(!bBackEdgeCorrect && !bLeftEdgeCorrect)   ||
		(!bFrontEdgeCorrect && !bLeftEdgeCorrect)  ||
		(!bBackEdgeCorrect && !bRightEdgeCorrect)  ||
		(!bFrontEdgeCorrect && !bBackEdgeCorrect && !bRightEdgeCorrect && !bLeftEdgeCorrect))
	{
		if (!bFrontEdgeCorrect && !bRightEdgeCorrect)
		{
			nLocalFront	=	FRONT;
			nLocalBack	=	BACK;
			nLocalRight	=	RIGHT;
			nLocalLeft	=	LEFT;
			
		}
		else if (!bBackEdgeCorrect && !bLeftEdgeCorrect)
		{
			nLocalFront	=	BACK;
			nLocalBack	=	FRONT;
			nLocalRight	=	LEFT;
			nLocalLeft	=	RIGHT;
		}
		else if (!bFrontEdgeCorrect && !bLeftEdgeCorrect)
		{
			nLocalFront	=	LEFT;
			nLocalBack	=	RIGHT;
			nLocalRight	=	FRONT;
			nLocalLeft	=	BACK;
		}
		else if (!bBackEdgeCorrect && !bRightEdgeCorrect)
		{
			nLocalFront	=	RIGHT;
			nLocalBack	=	LEFT;
			nLocalRight	=	BACK;
			nLocalLeft	=	LEFT;
		}
		//FURU'R'F'
		CMove aMove;

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalFront;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalFront;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if ( (m_Squares[TOP][MIDDLEROW][LEFTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
		 (m_Squares[TOP][MIDDLEROW][RIGHTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
		 (m_Squares[TOP][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
		 (m_Squares[TOP][BOTTOMROW][MIDDLECOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal))
	{
		return STEPFINISHED;
	}
	return m_nFaceToSolve;
}

int CRubiksCube::FixTopEdges()
{
	const int nCorrectBackEdge = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(BACK)};
	
	const int nCorrectFrontEdge = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(FRONT)};
	
	const int nCorrectLeftEdge = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(LEFT)};
	
	const int nCorrectRightEdge = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(RIGHT)};

	const int nCurrentBackEdge = m_Squares[TOP][TOPROW][MIDDLECOL].m_nColorsSharedBySquare;
	const int nCurrentFrontEdge= m_Squares[TOP][BOTTOMROW][MIDDLECOL].m_nColorsSharedBySquare;
	const int nCurrentLeftEdge = m_Squares[TOP][MIDDLEROW][LEFTCOL].m_nColorsSharedBySquare;
	const int nCurrentRightEdge= m_Squares[TOP][MIDDLEROW][RIGHTCOL].m_nColorsSharedBySquare;

	if (	((nCurrentRightEdge == nCorrectLeftEdge) &&
			(nCurrentLeftEdge  == nCorrectFrontEdge)&&
			(nCurrentFrontEdge == nCorrectRightEdge))
						||
			((nCurrentBackEdge == nCorrectFrontEdge) &&
			(nCurrentFrontEdge== nCorrectRightEdge) &&
			(nCurrentRightEdge== nCorrectBackEdge))
						||
			((nCurrentLeftEdge == nCorrectRightEdge) &&
			 (nCurrentRightEdge== nCorrectBackEdge) &&
			 (nCurrentBackEdge == nCorrectLeftEdge))
						||
			((nCurrentFrontEdge ==nCorrectBackEdge) && 
			 (nCurrentBackEdge == nCorrectLeftEdge) &&
			 (nCurrentLeftEdge == nCorrectFrontEdge))		
		)
	{
		int nLocalRight;

		if (nCurrentRightEdge == nCorrectLeftEdge)
		{
			nLocalRight = RIGHT;
		}
		else if (nCurrentBackEdge == nCorrectFrontEdge)
		{
			nLocalRight = BACK;
		}
		else if (nCurrentLeftEdge == nCorrectRightEdge)
		{
			nLocalRight = LEFT;
		}
		else
		{
			nLocalRight = FRONT;
		}
		//R'U''RUR'UR
		CMove aMove;

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if (	((nCurrentLeftEdge == nCorrectRightEdge) && 
			(nCurrentRightEdge == nCorrectFrontEdge) && 
			(nCurrentFrontEdge == nCorrectLeftEdge))
					||
			((nCurrentFrontEdge == nCorrectBackEdge) && 
			 (nCurrentBackEdge == nCorrectRightEdge) && 
			 (nCurrentRightEdge== nCorrectFrontEdge))
					||
			((nCurrentRightEdge == nCorrectLeftEdge) &&
			 (nCurrentLeftEdge == nCorrectBackEdge) &&
			 (nCurrentBackEdge == nCorrectRightEdge))
					||
			((nCurrentBackEdge == nCorrectFrontEdge) && 
			 (nCurrentFrontEdge== nCorrectLeftEdge) && 
			 (nCurrentLeftEdge == nCorrectBackEdge))		
		)
	{
		int nLocalRight;

		if (nCurrentLeftEdge == nCorrectRightEdge)
		{
			nLocalRight = RIGHT;
		}
		else if (nCurrentFrontEdge == nCorrectBackEdge)
		{
			nLocalRight = BACK;
		}
		else if (nCurrentRightEdge == nCorrectLeftEdge)
		{
			nLocalRight = LEFT;
		}
		else
		{
			nLocalRight = FRONT;
		}
		//R'U'RU'R'U''R
		CMove aMove;
		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if (	((nCurrentBackEdge == nCorrectLeftEdge) ||
			(nCurrentLeftEdge == nCorrectBackEdge))
						||
			((nCurrentLeftEdge == nCorrectFrontEdge) ||
			 (nCurrentFrontEdge== nCorrectLeftEdge))
						||
			((nCurrentFrontEdge == nCorrectRightEdge) ||
			 (nCurrentRightEdge == nCorrectFrontEdge))
						||
			((nCurrentRightEdge == nCorrectBackEdge) ||
			 (nCurrentBackEdge == nCorrectRightEdge))		
		)
	{//UR'U''RUR'UR
		int nLocalRight;
		
		if (nCurrentBackEdge == nCorrectLeftEdge)
		{
			nLocalRight = RIGHT;
		}
		else if (nCurrentLeftEdge == nCorrectFrontEdge)
		{
			nLocalRight = BACK;
		}
		else if (nCurrentFrontEdge == nCorrectRightEdge)
		{
			nLocalRight = LEFT;
		}
		else
		{
			nLocalRight = FRONT;
		}

		CMove aMove;
		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if (	((nCurrentFrontEdge == nCorrectBackEdge) && 
			(nCurrentBackEdge == nCorrectFrontEdge))
					||
			((nCurrentLeftEdge == nCorrectRightEdge) && 
			 (nCurrentRightEdge== nCorrectLeftEdge))		
		)
	{//R'U''RUR'URU' 
	 //R'U''RUR'UR
		int nLocalRight = RIGHT;

		if (nCurrentFrontEdge == nCorrectBackEdge)
		{
			nLocalRight = RIGHT;
		}
		else
		{
			nLocalRight = BACK;
		}

		CMove aMove;
		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if ( (m_Squares[FRONT][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[FRONT][MIDDLEROW][MIDDLECOL].m_nColorVal) && 
		 (m_Squares[RIGHT][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[RIGHT][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
		 (m_Squares[LEFT][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[LEFT][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
		 (m_Squares[BACK][TOPROW][MIDDLECOL].m_nColorVal == m_Squares[BACK][MIDDLEROW][MIDDLECOL].m_nColorVal))
	{
		return STEPFINISHED;
	}

	return m_nFaceToSolve;
}

int CRubiksCube::MovingTopCorners()
{
	const int nCorrectFrontRightCorner = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(RIGHT)};
	
	const int nCorrectFrontLeftCorner = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(LEFT)};
	
	const int nCorrectBackRightCorner =
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(BACK) | GetSharedColorValueOfFace(RIGHT)};
	
	const int nCorrectBackLeftCorner = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(BACK) | GetSharedColorValueOfFace(LEFT)};

	const int nCurrentFrontRightCorner	=	m_Squares[FRONT][TOPROW][RIGHTCOL].m_nColorsSharedBySquare;
	const int nCurrentFrontLeftCorner	=	m_Squares[FRONT][TOPROW][LEFTCOL].m_nColorsSharedBySquare;
	const int nCurrentBackRightCorner	=	m_Squares[BACK][TOPROW][LEFTCOL].m_nColorsSharedBySquare;
	const int nCurrentBackLeftCorner	=	m_Squares[BACK][TOPROW][RIGHTCOL].m_nColorsSharedBySquare;

	int nLocalFront	=	FRONT;
	int nLocalLeft	=	LEFT;
	int nLocalRight	=	RIGHT;
	int nLocalBack	=	BACK;

	if (	((nCurrentBackRightCorner == nCorrectFrontLeftCorner) && 
			 (nCurrentFrontLeftCorner == nCorrectFrontRightCorner) &&
			 (nCurrentFrontRightCorner== nCorrectBackRightCorner))
								||
			((nCurrentBackLeftCorner == nCorrectFrontRightCorner) &&
			 (nCurrentFrontRightCorner==nCorrectBackRightCorner) &&
			 (nCurrentBackRightCorner ==nCorrectBackLeftCorner))
								||
			((nCurrentFrontLeftCorner == nCorrectBackRightCorner) &&
			 (nCurrentBackRightCorner == nCorrectBackLeftCorner) &&
			 (nCurrentBackLeftCorner == nCorrectFrontLeftCorner))
								||
			((nCurrentFrontRightCorner == nCorrectBackLeftCorner) &&
			 (nCurrentBackLeftCorner == nCorrectFrontLeftCorner) &&
			 (nCurrentFrontLeftCorner== nCorrectFrontRightCorner))
		)
	{
		if (nCurrentBackRightCorner == nCorrectFrontLeftCorner)
		{
			nLocalFront	=	FRONT;
			nLocalLeft	=	LEFT;
			nLocalRight	=	RIGHT;
			nLocalBack	=	BACK;
		}
		else if (nCurrentBackLeftCorner == nCorrectFrontRightCorner)
		{
			nLocalFront	=	RIGHT;
			nLocalLeft	=	FRONT;
			nLocalRight	=	BACK;
			nLocalBack	=	LEFT;
		}
		else if (nCurrentFrontLeftCorner == nCorrectBackRightCorner)
		{
			nLocalFront	=	BACK;
			nLocalLeft	=	RIGHT;
			nLocalRight	=	LEFT;
			nLocalBack	=	FRONT;
		}
		else
		{
			nLocalFront	=	LEFT;
			nLocalLeft	=	BACK;
			nLocalRight	=	FRONT;
			nLocalBack	=	RIGHT;
		}
		//ULU'R'UL'U'R
		CMove aMove;

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalLeft;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalLeft;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if (		((nCurrentBackRightCorner == nCorrectFrontRightCorner) &&
				 (nCurrentFrontRightCorner== nCorrectFrontLeftCorner) &&
				 (nCurrentFrontLeftCorner == nCorrectBackRightCorner))
									||
				((nCurrentBackLeftCorner == nCorrectBackRightCorner) &&
				 (nCurrentBackRightCorner== nCorrectFrontRightCorner) &&
				 (nCurrentFrontRightCorner==nCorrectBackLeftCorner))
									||
				((nCurrentFrontLeftCorner==nCorrectBackLeftCorner) &&
				 (nCurrentBackLeftCorner ==nCorrectBackRightCorner) &&
				 (nCurrentBackRightCorner==nCorrectFrontLeftCorner))
									||
				((nCurrentFrontRightCorner == nCorrectFrontLeftCorner) &&
				 (nCurrentFrontLeftCorner == nCorrectBackLeftCorner) &&
				 (nCurrentBackLeftCorner == nCorrectFrontRightCorner))
		)
	{//R'ULU'RUL'U'

		if (nCurrentBackRightCorner == nCorrectFrontRightCorner)
		{
			nLocalFront	=	FRONT;
			nLocalLeft	=	LEFT;
			nLocalRight	=	RIGHT;
			nLocalBack	=	BACK;
		}
		else if (nCurrentBackLeftCorner == nCorrectBackRightCorner)
		{
			nLocalFront	=	RIGHT;
			nLocalLeft	=	FRONT;
			nLocalRight	=	BACK;
			nLocalBack	=	LEFT;
		}
		else if(nCurrentFrontLeftCorner == nCorrectBackLeftCorner)
		{
			nLocalFront	=	BACK;
			nLocalLeft	=	RIGHT;
			nLocalRight	=	LEFT;
			nLocalBack	=	FRONT;
		}
		else
		{
			nLocalFront	=	LEFT;
			nLocalLeft	=	BACK;
			nLocalRight	=	FRONT;
			nLocalBack	=	RIGHT;
		}

		CMove aMove;
		
		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalLeft;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalLeft;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if (		(((nCurrentBackRightCorner == nCorrectBackLeftCorner) ||
				 (nCurrentBackLeftCorner == nCorrectBackRightCorner))
										||
				((nCurrentFrontRightCorner == nCorrectFrontLeftCorner) ||
				 (nCurrentFrontLeftCorner == nCorrectFrontRightCorner)))

										||

				(((nCurrentFrontLeftCorner == nCorrectBackLeftCorner) ||
				 (nCurrentBackLeftCorner == nCorrectFrontLeftCorner))
										||
				((nCurrentFrontRightCorner == nCorrectBackRightCorner) ||
				 (nCurrentBackRightCorner == nCorrectFrontRightCorner)))
		)
	{//FRUR'U'RUR'U'RUR'U'F'

		if (((nCurrentBackRightCorner == nCorrectBackLeftCorner) ||
				 (nCurrentBackLeftCorner == nCorrectBackRightCorner))
										||
				((nCurrentFrontRightCorner == nCorrectFrontLeftCorner) ||
				 (nCurrentFrontLeftCorner == nCorrectFrontRightCorner)))
		{
			nLocalFront	=	FRONT;
			nLocalLeft	=	LEFT;
			nLocalRight	=	RIGHT;
			nLocalBack	=	BACK;
		}
		else
		{
			nLocalFront	=	RIGHT;
			nLocalLeft	=	FRONT;
			nLocalRight	=	BACK;
			nLocalBack	=	LEFT;
		}

		CMove aMove;

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalFront;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);		

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);		

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);		

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalRight;
		m_ListOfMoves.push_back(aMove);		

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	nLocalFront;
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;

	}

	if (((nCurrentFrontRightCorner == nCorrectBackLeftCorner) ||
		 (nCurrentFrontLeftCorner == nCorrectBackRightCorner) ||
		 (nCurrentBackLeftCorner == nCorrectFrontRightCorner) ||
		 (nCurrentBackRightCorner == nCorrectFrontLeftCorner))
		 )
	{
		CMove aMove;

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	RIGHT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	LEFT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	BOTTOM;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	RIGHT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	LEFT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	RIGHT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	LEFT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	BOTTOM;
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	RIGHT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	LEFT;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_ListOfMoves.push_back(aMove);

		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	if ((nCurrentFrontRightCorner == nCorrectFrontRightCorner) &&
		(nCurrentFrontLeftCorner == nCorrectFrontLeftCorner) &&
		(nCurrentBackRightCorner == nCorrectBackRightCorner) &&
		(nCurrentBackLeftCorner == nCorrectBackLeftCorner)
		)
	{
		return STEPFINISHED;
	}

	return m_nFaceToSolve;
}

int CRubiksCube::FixTopCorners()
{
	// 0 = not rotations yet
	// 1 = clockwise
	// 2 = counterclockwise
	const int UNDETERMINED		=	0;
	const int CLOCKWISE			=	1;
	const int COUNTERCLOCKWISE	=	2;

	static int nNextCornerRotation = UNDETERMINED;
	static bool bSquareNeedsCW = false;
	static bool bSquareNeedsCCW= false;
	static int nCurrCorner = 0;

	static const int Corners[4] = 
	{GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(RIGHT),
	 GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(FRONT) | GetSharedColorValueOfFace(LEFT),
	 GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(BACK)  | GetSharedColorValueOfFace(RIGHT),
	 GetSharedColorValueOfFace(TOP) | GetSharedColorValueOfFace(BACK)  | GetSharedColorValueOfFace(LEFT)};

	//get the current corner to solve into the FRONT UPPER RIGHT corner
	if (m_Squares[FRONT][TOPROW][RIGHTCOL].m_nColorsSharedBySquare != Corners[nCurrCorner])
	{
		CMove aMove;
		aMove.m_bClockwise	=	true;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}

	//if the square we are trying to solve is in the upper right corner
	//see if it is already solved

	if (m_Squares[TOP][BOTTOMROW][RIGHTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
	{
		//check to see if the whole top is complete, if not, try the next corner
		if ((m_Squares[TOP][TOPROW][RIGHTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
			(m_Squares[TOP][TOPROW][LEFTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
			(m_Squares[TOP][BOTTOMROW][RIGHTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal) &&
			(m_Squares[TOP][BOTTOMROW][LEFTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
			)
		{
			nNextCornerRotation = UNDETERMINED;
			bSquareNeedsCW = false;
			bSquareNeedsCCW= false;
			nCurrCorner = 0;
			return STEPFINISHED;
		}
		nCurrCorner++;
		if (nCurrCorner >= 4)
		{
			nNextCornerRotation = UNDETERMINED;
			bSquareNeedsCW = false;
			bSquareNeedsCCW= false;
			nCurrCorner = 0;
			return STEPFINISHED;
		}
		return nCurrCorner;
	}

	if (UNDETERMINED == nNextCornerRotation)
	{//CLOCKWISE
		if (m_Squares[FRONT][TOPROW][RIGHTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			RotateCornerClockwise(FRONT, RIGHT, LEFT, BACK);
			nNextCornerRotation = COUNTERCLOCKWISE;
			m_nPrevCubeState = m_nCubeState;
			m_nCubeState = ROTATINGFACE;		
			bSquareNeedsCW = false;
			bSquareNeedsCCW= true;
			return m_nFaceToSolve;
		}
		else if (m_Squares[RIGHT][TOPROW][LEFTCOL].m_nColorVal == m_Squares[TOP][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			RotateCornerCounterClockwise(FRONT, RIGHT, LEFT, BACK);
			nNextCornerRotation = CLOCKWISE;
			m_nPrevCubeState = m_nCubeState;
			m_nCubeState = ROTATINGFACE;		
			bSquareNeedsCW = true;
			bSquareNeedsCCW= false;
			return m_nFaceToSolve;
		}
	}

	if (CLOCKWISE == nNextCornerRotation)
	{
		RotateCornerClockwise(FRONT, RIGHT, LEFT, BACK);
		nNextCornerRotation = UNDETERMINED;
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		bSquareNeedsCW = false;
		bSquareNeedsCCW= false;
		return m_nFaceToSolve;
	}

	if (COUNTERCLOCKWISE == nNextCornerRotation)
	{
		RotateCornerCounterClockwise(FRONT, RIGHT, LEFT, BACK);
		nNextCornerRotation = UNDETERMINED;
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		bSquareNeedsCW = false;
		bSquareNeedsCCW= false;
		return m_nFaceToSolve;
	}

	return m_nFaceToSolve;
}

int CRubiksCube::MovingTopFace()
{
	if (CubeIsSolved())
	{
		return STEPFINISHED;
	}
	else
	{
		CMove aMove;
		aMove.m_bClockwise	=	false;
		aMove.m_nFaceToMove	=	TOP;
		m_ListOfMoves.push_back(aMove);
		m_nPrevCubeState = m_nCubeState;
		m_nCubeState = ROTATINGFACE;		
		return m_nFaceToSolve;
	}
}

void CRubiksCube::RotateCornerClockwise(int nLocalFront, int nLocalRight, int nLocalLeft, int nLocalBack)
{//FD''F'R'D''R
	CMove aMove;

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	nLocalFront;
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	BOTTOM;
	m_ListOfMoves.push_back(aMove);
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	false;
	aMove.m_nFaceToMove	=	nLocalFront;
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	false;
	aMove.m_nFaceToMove	=	nLocalRight;
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	BOTTOM;
	m_ListOfMoves.push_back(aMove);
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	nLocalRight;
	m_ListOfMoves.push_back(aMove);
}

void CRubiksCube::RotateCornerCounterClockwise(int nLocalFront, int nLocalRight, int nLocalLeft, int nLocalBack)
{//R'D''RFD''F'
	CMove aMove;

	aMove.m_bClockwise	=	false;
	aMove.m_nFaceToMove	=	nLocalRight;
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	BOTTOM;
	m_ListOfMoves.push_back(aMove);
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	nLocalRight;
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	nLocalFront;
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	true;
	aMove.m_nFaceToMove	=	BOTTOM;
	m_ListOfMoves.push_back(aMove);
	m_ListOfMoves.push_back(aMove);

	aMove.m_bClockwise	=	false;
	aMove.m_nFaceToMove	=	nLocalFront;
	m_ListOfMoves.push_back(aMove);
}

int CRubiksCube::GetSharedColorValueOfFace(int Face)
{
	switch(Face)
	{
	case FRONT:
		return WHITEFACE;
	case RIGHT:
		return GREENFACE;
	case TOP:
		return REDFACE;
	case LEFT:
		return BLUEFACE;
	case BACK:
		return YELLOWFACE;
	case BOTTOM:
		return ORANGEFACE;
	default:
		return BLACK;
	}
}

bool CRubiksCube::SquareIsInCorrectPosition(int NumOfFaces, int* Face, int* Row, int* Col)
{
	for (int x = 0; x < NumOfFaces; x++)
	{
		if (m_Squares[Face[x]][Row[x]][Col[x]].m_nColorVal != m_Squares[Face[x]][MIDDLEROW][MIDDLECOL].m_nColorVal)
		{
			return false;
		}
	}

	return true;
}

void CRubiksCube::GenerateSolution()
{
	int CurrFace, CurrRow, CurrCol;
	m_Solution.clear();
	m_ListOfMoves.clear();
//copy current state of cube to internal representation
	for (CurrFace = FRONT; CurrFace <= BOTTOM; CurrFace++)
	{
		for (CurrRow = 0; CurrRow < 3; CurrRow++)
		{
			for (CurrCol = 0; CurrCol < 3; CurrCol++)
			{
				m_InternalSquares[CurrFace][CurrCol][CurrRow] = m_Squares[CurrFace][CurrCol][CurrRow];
			}
		}
	}

	while (m_nCubeState != SOLVED)
	{
		switch(m_nCubeState)
		{
		case ROTATINGFACE:
			{
				for (list<CMove>::iterator it = m_ListOfMoves.begin(); it != m_ListOfMoves.end(); it++)
				{
					RotateFace(it->m_nFaceToMove, it->m_bClockwise);
					m_Solution.push_back(*it);
				}

				m_ListOfMoves.clear();

				if (m_nPrevCubeState == -1)
				{
					m_nCubeState  = MAKEBOTTOMCROSS;
				}
				else
				{
					m_nCubeState  = m_nPrevCubeState;
				}
			}
			break;

		case MAKEBOTTOMCROSS:
			{
				if(GetNextMoveMakingBottomCross() == STEPFINISHED)
				{
					m_nCubeState = MAKINGBOTTOMCORNERS;
					m_nFaceToSolve = BOTTOM;
				}
			}
			break;

		case MAKINGBOTTOMCORNERS:
			{
				if(GetNextMoveMakingBottomCorners() == STEPFINISHED)
				{
					m_nCubeState = SOLVINGEDGES;
					m_nFaceToSolve = FRONT;
				}
			}
			break;
		case SOLVINGEDGES:
			{
				if(GetNextMoveSolvingEdges() == STEPFINISHED)
				{
					m_nCubeState = MOVINGWORKINGCORNER;
				}
			}
			break;

		case MOVINGWORKINGCORNER:
			{
				if(MovingWorkingCorner() == STEPFINISHED)
				{
					m_nCubeState = FIXWORKINGCORNER;
				}
			}
			break;
		
		case FIXWORKINGCORNER:
			{
				if(FixWorkingCorner() == STEPFINISHED)
				{
					m_nCubeState = FIXWORKINGEDGE;
					m_nFaceToSolve = FRONT;
				}
			}
			break;

		case FIXWORKINGEDGE:
			{
				if(FixWorkingEdge() == STEPFINISHED)
				{
					m_nCubeState = MAKETOPCROSS;
				}
			}
			break;

		case MAKETOPCROSS:
			{
				if(MakeTopCross() == STEPFINISHED)
				{
					m_nCubeState = FIXTOPEDGES;
				}
			}
			break;

		case FIXTOPEDGES:
			{
				if(FixTopEdges() == STEPFINISHED)
				{
					m_nCubeState = MOVINGTOPCORNERS;
					m_nFaceToSolve = FRONT;
				}
			}
			break;

		case MOVINGTOPCORNERS:
			{
				if(MovingTopCorners() == STEPFINISHED)
				{
					m_nCubeState = FIXTOPCORNERS;
				}
			}
			break;

		case FIXTOPCORNERS:
			{
				if(FixTopCorners() == STEPFINISHED)
				{
					m_nCubeState = MOVINGTOPFACE;
				}
			}
			break;

		case MOVINGTOPFACE:
			{
				if(MovingTopFace() == STEPFINISHED)
				{
					m_nCubeState = SOLVED;
				}
			}
			break;
		case SOLVED:
			break;
		}
	}

//copy internal representation back to original state
	for (CurrFace = FRONT; CurrFace <= BOTTOM; CurrFace++)
	{
		for (CurrRow = 0; CurrRow < 3; CurrRow++)
		{
			for (CurrCol = 0; CurrCol < 3; CurrCol++)
			{
				m_Squares[CurrFace][CurrCol][CurrRow] = m_InternalSquares[CurrFace][CurrCol][CurrRow];
			}
		}
	}
	//reassign the moves list, and set the state to rotating face
	m_nCubeState = ROTATINGFACE;
	m_nPrevCubeState = ROTATINGFACE;
//Consolidate MovesList
	ConsolidateSolution();
	//UNCOMMENT THIS LINE TO BRING THE ANIMATION OF THE CUBE SOLVING
	//m_ListOfMoves = m_Solution;
}

void CRubiksCube::ConsolidateSolution()
{
	m_ListOfMoves.clear();

#ifdef _DEBUG
	//m_Solution.clear();
	//ReadListFromFile();
#endif

	list<CMove>::iterator it = m_Solution.begin();

	//using the inline function AssignEnd, we make sure we arent past the end of the list
	//or AT the end, and trying to assing invalid values.  if our iterator at
	//any time assign an invalid value to the move, it will be caught in the bool variable
	//this is certainly NOT the best way to do this, but it surely was easy to code
	//and easy to understand.  We can now simply run right through the whole list
	//and not have to worry about iterating through size - 4, or whatever

	while (it != m_Solution.end())
	{
		bool bEnd1 = false;
		bool bEnd2 = false;
		bool bEnd3 = false;
		bool bEnd4 = false;

		CMove Move1, Move2, Move3, Move4;
		Move1 = *it;
	
		bEnd1 = AssignEnd(&it);

		it++;	//pointing to the second
		Move2 = *it;

		bEnd2 = AssignEnd(&it);

		it++;	//pointing to the third
		Move3 = *it;

		bEnd3 = AssignEnd(&it);

		it++;	//poiting to the fourth
		Move4 = *it;

		bEnd4 = AssignEnd(&it);

		//if the current move and following 3 moves are the same, move ahead 
		if (Move1 == Move2 && Move1 == Move3 && Move1 == Move4 && (!bEnd1 && !bEnd2 && !bEnd3 && !bEnd4))
		{
			//move the iterator to the next move to check
			it++;
		}
		else
		{
			//this is not the start of a series of 4, so include
			m_ListOfMoves.push_back(Move1);
			//move the iterator back to the next move to check
			it--;	//pointing to the third
			it--;	//pointing to the second
		}
	}

	//set the solution to the newly shortened list, without any
	//faces turning in the same direction 4 times in a row
	m_Solution = m_ListOfMoves;
	//wipe out the moves list
	m_ListOfMoves.clear();

	it = m_Solution.begin();
	while (it != m_Solution.end())
	{
		bool bEnd1 = false;
		bool bEnd2 = false;
		bool bEnd3 = false;
		CMove Move1, Move2, Move3;
		Move1 = *it;

		bEnd1 = AssignEnd(&it);

		it++;	//pointing to the second
		Move2 = *it;

		bEnd2 = AssignEnd(&it);

		it++;	//pointing to the third
		Move3 = *it;

		bEnd3 = AssignEnd(&it);

		//if the current move and following 2 moves are the same, reverse the direction, and move ahead 
		if (Move1 == Move2 && Move1 == Move3 && (!bEnd1 && !bEnd2 && !bEnd3))
		{
			Move1.m_bClockwise = !(Move1.m_bClockwise);
			m_ListOfMoves.push_back(Move1);
			//move the iterator to the next move to check
			it++;
		}
		else
		{
			//this is not the start of a series of 3, so include
			m_ListOfMoves.push_back(Move1);
			//move the iterator back to the next move to check
			it--;	//pointing to the second
		}
	}

	m_Solution = m_ListOfMoves;
	//wipe out the moves list
	m_ListOfMoves.clear();

	it = m_Solution.begin();

	while (it != m_Solution.end())
	{
		bool bEnd1;
		bool bEnd2;
		CMove Move1, Move2;
		Move1 = *it;

		bEnd1 = AssignEnd(&it);

		it++;	//pointing to the second
		Move2 = *it;

		bEnd2 = AssignEnd(&it);

		//if the current move and the next move are the same, skip it
		if (Move1.IsOpposite(Move2) && (!bEnd1 && !bEnd2))
		{
			it++;
		}
		else
		{
			//this is not a move, and then the opposite, so include it
			m_ListOfMoves.push_back(Move1);
		}
	}


#ifdef _DEBUG
	OutputMoveList();
#endif

}

#ifdef _DEBUG
void CRubiksCube::OutputMoveList()
{
	FILE* fptrMoves = fopen("C:\\windows\\desktop\\MovesList.txt","w");
	if (!fptrMoves)
		return;

	char* Faces[6] = {"Front", "Right", "Top", "Back", "Left", "Bottom"};

	list<CMove>::iterator it = m_ListOfMoves.begin();

	while (it != m_ListOfMoves.end())
	{
		char buff[1024];
		sprintf(buff,"%s\t", Faces[it->m_nFaceToMove]);
		strcat(buff, ((it->m_bClockwise) ? "Clockwise" : "CounterClockwise"));
		fprintf(fptrMoves,"%s\n",buff);
		it++;
	}

	fclose(fptrMoves);
}
#endif

#ifdef _DEBUG
void CRubiksCube::ReadListFromFile()
{
	FILE* Infile = fopen("c:\\windows\\desktop\\MovesList_HAS_AN_OPPOSITE.txt","r");
	if (!Infile)
		return;

	char line[1024];
	char face[100];
	char direction[100];

	while (fgets(line,1024,Infile))
	{
		CMove aMove;
		sscanf(line,"%s\t%s\n",face,direction);

		if (!strcmp(face,"Top"))
		{
			aMove.m_nFaceToMove = TOP;
		}
		else if (!strcmp(face,"Right"))
		{
			aMove.m_nFaceToMove = RIGHT;
		}
		else if (!strcmp(face,"Bottom"))
		{
			aMove.m_nFaceToMove = BOTTOM;
		}
		else if (!strcmp(face,"Left"))
		{
			aMove.m_nFaceToMove = LEFT;
		}
		else if (!strcmp(face,"Back"))
		{
			aMove.m_nFaceToMove = BACK;
		}
		else if (!strcmp(face,"Front"))
		{
			aMove.m_nFaceToMove = FRONT;
		}

		if (!strcmp(direction, "Clockwise"))
		{
			aMove.m_bClockwise = true;
		}
		else
		{
			aMove.m_bClockwise = false;
		}

		m_Solution.push_back(aMove);
	}

	fclose(Infile);
}
#endif