// RubiksCube.h: interface for the CRubiksCube class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RUBIKSCUBE_H__9975DEC9_D643_11D6_8EAA_AE4A8565A828__INCLUDED_)
#define AFX_RUBIKSCUBE_H__9975DEC9_D643_11D6_8EAA_AE4A8565A828__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Square.h"
#include "Move.h"
#include <list>
using namespace std;

class CRubiksCube  
{
public:
	CRubiksCube();
	virtual ~CRubiksCube();
	void Draw();
	void ResetCube();
	void Update();
	void GenerateSolution();
	void SetAlpha(float fAlpha) {m_fAlpha = fAlpha;}
	float GetAlpha() {return m_fAlpha;}
	void SetRotatingFace(int nRotatingFace) {m_nRotatingFace = nRotatingFace;}
	int GetRotatingFace() {return m_nRotatingFace;}
	void SetAutoSolve(bool bAutoSolve) {m_bAutoSolve = bAutoSolve;}
	bool IsAutoSolve() {return m_bAutoSolve;}
	float GetXRotation () {return m_fXRotation;}
	float GetYRotation () {return m_fYRotation;}
	float GetZRotation () {return m_fZRotation;}
	void SetXRotation (float fXRotation) {m_fXRotation = fXRotation;}
	void SetYRotation (float fYRotation) {m_fYRotation = fYRotation;}
	void SetZRotation (float fZRotation) {m_fZRotation = fZRotation;}
	float GetRotationSpeed() {return m_fRotationSpeed;}
	void SetRotationSpeed(float fRotationSpeed) {m_fRotationSpeed = fRotationSpeed;}
	void SetCubeState(int nCubeState) {m_nCubeState = nCubeState;}
	int GetCubeState() {return m_nCubeState;}
	void SetPrevCubeState(int nPrevCubeState) {m_nPrevCubeState = nPrevCubeState;}
	
	list<CMove> m_ListOfMoves;
	list<CMove>::iterator m_ListIterator;

private:

#ifdef _DEBUG
	void OutputCube();
	void OutputMoveList();
	void ReadListFromFile();
#endif

	CSquare m_InternalSquares[6][3][3];
	CSquare	m_Squares[6][3][3];
	list<CMove> m_Solution;
	int		m_nCubeState;
	int		m_nPrevCubeState;
	bool	m_bClockwise;
	bool	m_bAutoSolve;
	float	m_fXRotation;
	float	m_fYRotation;
	float	m_fZRotation;
	float	m_fRotationSpeed;
	static	const	int		m_nSolvedCube[6][3][3];
	static	const	float	m_fCubieOffsetFromCenter;
	static	const	int		WHITEFACE;
	static	const	int		BLUEFACE;
	static	const	int		REDFACE;
	static	const	int		ORANGEFACE;
	static	const	int		YELLOWFACE;
	static	const	int		GREENFACE;
	int		m_nFaceToSolve;
	float	m_fvPosition[3];
	float	m_fXFaceRotation;
	float	m_fYFaceRotation;
	float	m_fZFaceRotation;
	float	m_fAlpha;
	int		m_nRotatingFace;

	void ConsolidateSolution();
	inline bool AssignEnd(list<CMove>::iterator* it) {return (*it == m_Solution.end());}
	void ScrambleCube();
	void ResetDrawnStatus();
	void DrawFront(int currFace,int row, int col);
	void DrawLeft(int currFace,int row, int col);
	void DrawRight(int currFace,int row, int col);
	void DrawBack(int currFace,int row, int col);
	void DrawTop(int currFace,int row, int col);
	void DrawBottom(int currFace,int row, int col);
	void RotateFront(int currFace, bool bClockwise);
	void RotateLeft(int currFace, bool bClockwise);
	void RotateBack(int currFace, bool bClockwise);
	void RotateRight(int currFace, bool bClockwise);
	void RotateBottom(int currFace, bool bClockwise);
	void RotateTop(int currFace, bool bClockwise);
	void RotateFace(int Face, bool Direction);
	int GetSharedColorValueOfFace(int Face);
	int GetNextMoveMakingBottomCross();	
	int GetNextMoveMakingBottomCorners();
	int GetNextMoveSolvingEdges();
	int MovingWorkingCorner();
	int FixWorkingCorner();
	int FixWorkingEdge();
	int MakeTopCross();
	int FixTopEdges();
	int MovingTopCorners();
	int FixTopCorners();
	int MovingTopFace();
	void RotateCornerClockwise(int nLocalFront, int nLocalRight, int nLocalLeft, int nLocalBack);
	void RotateCornerCounterClockwise(int nLocalFront, int nLocalRight, int nLocalLeft, int nLocalBack);
	bool CubeIsSolved();
	bool SquareIsInCorrectPosition(int NumOfFaces, int* Face, int* Row, int* Col);
	inline int	 GetSharedColors(int currFace, int row, int col) {return m_Squares[currFace][row][col].m_nColorsSharedBySquare;};
	int	GetPieceCoords(int Colors, int* Face, int* Row, int* Col);
};

#endif // !defined(AFX_RUBIKSCUBE_H__9975DEC9_D643_11D6_8EAA_AE4A8565A828__INCLUDED_)
